@@ -27,10 +27,24 @@
 #ifdef HAVE_LIMITS_H
 #include <limits.h>
 #endif /* HAVE_LIMITS_H */
-#include "regex.h"
+
+#ifndef UINT_MAX
+# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
+#endif
+
+#ifndef INT_MAX
+# define INT_MAX ((int) (UINT_MAX >> 1))
+#endif
+
+#if WITH_REGEX
+# include <regex.h>
+#else
+# include <rx.h>
+#endif
 #include "system.h"
 #include "version.h"
 #include "error.h"
+#include "xstrtoul.h"
 
 #ifdef STDC_HEADERS
 #include <stdlib.h>
@@ -43,16 +57,6 @@ char *realloc ();
 #define MAX(a,b) (((a) > (b)) ? (a) : (b))
 #endif
 
-int safe_read ();
-
-static char *xrealloc ();
-static char *xmalloc ();
-static void cleanup ();
-static void close_output_file ();
-static void create_output_file ();
-static void save_line_to_file ();
-static void usage ();
-
 #ifndef TRUE
 #define FALSE 0
 #define TRUE 1
@@ -129,6 +133,14 @@ struct buffer_record
   struct buffer_record *next;
 };
 
+int safe_read ();
+
+static void close_output_file __P ((void));
+static void create_output_file __P ((void));
+static void delete_all_files __P ((void));
+static void save_line_to_file __P ((const struct cstring *line));
+static void usage __P ((int status));
+
 /* The name this program was run with. */
 char *program_name;
 
@@ -206,10 +218,10 @@ static struct control *controls;
 /* Number of elements in `controls'. */
 static unsigned int control_used;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output then exit.  */
+/* If nonzero, print the version on standard output then exit.  */
 static int show_version;
 
 static struct option const longopts[] =
@@ -226,11 +238,47 @@ static struct option const longopts[] =
   {NULL, 0, NULL, 0}
 };
 
+/* Optionally remove files created so far; then exit.
+   Called when an error detected. */
+
+static void
+cleanup (void)
+{
+  if (output_stream)
+    close_output_file ();
+
+  if (remove_files)
+    delete_all_files ();
+}
+
+static void
+cleanup_fatal (void)
+{
+  cleanup ();
+  exit (1);
+}
+
+static RETSIGTYPE
+interrupt_handler (int sig)
+{
+#ifdef SA_INTERRUPT
+  struct sigaction sigact;
+
+  sigact.sa_handler = SIG_DFL;
+  sigemptyset (&sigact.sa_mask);
+  sigact.sa_flags = 0;
+  sigaction (sig, &sigact, NULL);
+#else				/* !SA_INTERRUPT */
+  signal (sig, SIG_DFL);
+#endif				/* SA_INTERRUPT */
+  cleanup ();
+  kill (getpid (), sig);
+}
+
 /* Allocate N bytes of memory dynamically, with error checking.  */
 
 static char *
-xmalloc (n)
-     unsigned int n;
+xmalloc (unsigned int n)
 {
   char *p;
 
@@ -238,7 +286,7 @@ xmalloc (n)
   if (p == NULL)
     {
       error (0, 0, _("virtual memory exhausted"));
-      cleanup ();
+      cleanup_fatal ();
     }
   return p;
 }
@@ -249,9 +297,7 @@ xmalloc (n)
    If N is 0, run free and return NULL.  */
 
 static char *
-xrealloc (p, n)
-     char *p;
-     unsigned int n;
+xrealloc (char *p, unsigned int n)
 {
   if (p == NULL)
     return xmalloc (n);
@@ -264,7 +310,7 @@ xrealloc (p, n)
   if (p == NULL)
     {
       error (0, 0, _("virtual memory exhausted"));
-      cleanup ();
+      cleanup_fatal ();
     }
   return p;
 }
@@ -278,9 +324,7 @@ xrealloc (p, n)
    start of the new buffer. */
 
 static void
-save_to_hold_area (start, num)
-     char *start;
-     unsigned int num;
+save_to_hold_area (char *start, unsigned int num)
 {
   hold_area = start;
   hold_count = num;
@@ -290,9 +334,7 @@ save_to_hold_area (start, num)
    Return the number of chars read. */
 
 static int
-read_input (dest, max_n_bytes)
-     char *dest;
-     unsigned int max_n_bytes;
+read_input (char *dest, unsigned int max_n_bytes)
 {
   int bytes_read;
 
@@ -307,7 +349,7 @@ read_input (dest, max_n_bytes)
   if (bytes_read < 0)
     {
       error (0, errno, _("read error"));
-      cleanup ();
+      cleanup_fatal ();
     }
 
   return bytes_read;
@@ -316,8 +358,7 @@ read_input (dest, max_n_bytes)
 /* Initialize existing line record P. */
 
 static void
-clear_line_control (p)
-     struct line *p;
+clear_line_control (struct line *p)
 {
   p->used = 0;
   p->insert_index = 0;
@@ -327,8 +368,7 @@ clear_line_control (p)
 /* Initialize all line records in B. */
 
 static void
-clear_all_line_control (b)
-     struct buffer_record *b;
+clear_all_line_control (struct buffer_record *b)
 {
   struct line *l;
 
@@ -339,7 +379,7 @@ clear_all_line_control (b)
 /* Return a new, initialized line record. */
 
 static struct line *
-new_line_control ()
+new_line_control (void)
 {
   struct line *p;
 
@@ -355,10 +395,7 @@ new_line_control ()
    of length LINE_LEN in the large buffer, in the lines buffer of B. */
 
 static void
-keep_new_line (b, line_start, line_len)
-     struct buffer_record *b;
-     char *line_start;
-     int line_len;
+keep_new_line (struct buffer_record *b, char *line_start, int line_len)
 {
   struct line *l;
 
@@ -391,8 +428,7 @@ keep_new_line (b, line_start, line_len)
    the next buffer is filled. */
 
 static unsigned int
-record_line_starts (b)
-     struct buffer_record *b;
+record_line_starts (struct buffer_record *b)
 {
   char *line_start;		/* Start of current line. */
   char *line_end;		/* End of each line found. */
@@ -442,8 +478,7 @@ record_line_starts (b)
    an extra byte for safety. */
 
 static struct buffer_record *
-create_new_buffer (size)
-     unsigned int size;
+create_new_buffer (unsigned int size)
 {
   struct buffer_record *new_buffer;
 
@@ -462,8 +497,7 @@ create_new_buffer (size)
    least that size is currently free, use it, otherwise create a new one. */
 
 static struct buffer_record *
-get_new_buffer (min_size)
-     unsigned int min_size;
+get_new_buffer (unsigned int min_size)
 {
   struct buffer_record *p, *q;
   struct buffer_record *new_buffer; /* Buffer to return. */
@@ -513,8 +547,7 @@ get_new_buffer (min_size)
 /* Add buffer BUF to the list of free buffers. */
 
 static void
-free_buffer (buf)
-     struct buffer_record *buf;
+free_buffer (struct buffer_record *buf)
 {
   buf->next = free_list;
   free_list = buf;
@@ -524,8 +557,7 @@ free_buffer (buf)
    some data yet to be processed. */
 
 static void
-save_buffer (buf)
-     struct buffer_record *buf;
+save_buffer (struct buffer_record *buf)
 {
   struct buffer_record *p;
 
@@ -556,7 +588,7 @@ save_buffer (buf)
    (in which case end-of-file must have been encountered). */
 
 static boolean
-load_buffer ()
+load_buffer (void)
 {
   struct buffer_record *b;
   unsigned int bytes_wanted = START_SIZE; /* Minimum buffer size. */
@@ -607,7 +639,7 @@ load_buffer ()
 /* Return the line number of the first line that has not yet been retrieved. */
 
 static unsigned int
-get_first_line_in_buffer ()
+get_first_line_in_buffer (void)
 {
   if (head == NULL && !load_buffer ())
     error (1, errno, _("input disappeared"));
@@ -620,7 +652,7 @@ get_first_line_in_buffer ()
    Return NULL if there is no more input. */
 
 static struct cstring *
-remove_line ()
+remove_line (void)
 {
   struct cstring *line;		/* Return value. */
   struct line *l;		/* For convenience. */
@@ -658,8 +690,7 @@ remove_line ()
    Return a pointer to the line, or NULL if it is not found in the file. */
 
 static struct cstring *
-find_line (linenum)
-     unsigned int linenum;
+find_line (unsigned int linenum)
 {
   struct buffer_record *b;
 
@@ -696,7 +727,7 @@ find_line (linenum)
 /* Return TRUE if at least one more line is available for input. */
 
 static boolean
-no_more_lines ()
+no_more_lines (void)
 {
   return (find_line (current_line + 1) == NULL) ? TRUE : FALSE;
 }
@@ -704,8 +735,7 @@ no_more_lines ()
 /* Set the name of the input file to NAME and open it. */
 
 static void
-set_input_file (name)
-     char *name;
+set_input_file (const char *name)
 {
   if (!strcmp (name, "-"))
     input_desc = 0;
@@ -723,10 +753,7 @@ set_input_file (name)
    ARGNUM is the index in ARGV of the current pattern. */
 
 static void
-write_to_file (last_line, ignore, argnum)
-     unsigned int last_line;
-     boolean ignore;
-     int argnum;
+write_to_file (unsigned int last_line, boolean ignore, int argnum)
 {
   struct cstring *line;
   unsigned int first_line;		/* First available input line. */
@@ -738,7 +765,7 @@ write_to_file (last_line, ignore, argnum)
   if (first_line > last_line)
     {
       error (0, 0, _("%s: line number out of range"), global_argv[argnum]);
-      cleanup ();
+      cleanup_fatal ();
     }
 
   lines = last_line - first_line;
@@ -749,7 +776,7 @@ write_to_file (last_line, ignore, argnum)
       if (line == NULL)
 	{
 	  error (0, 0, _("%s: line number out of range"), global_argv[argnum]);
-	  cleanup ();
+	  cleanup_fatal ();
 	}
       if (!ignore)
 	save_line_to_file (line);
@@ -759,7 +786,7 @@ write_to_file (last_line, ignore, argnum)
 /* Output any lines left after all regexps have been processed. */
 
 static void
-dump_rest_of_file ()
+dump_rest_of_file (void)
 {
   struct cstring *line;
 
@@ -771,9 +798,7 @@ dump_rest_of_file ()
    on iteration REPETITION if nonzero. */
 
 static void
-handle_line_error (p, repetition)
-     struct control *p;
-     int repetition;
+handle_line_error (const struct control *p, int repetition)
 {
   fprintf (stderr, _("%s: `%d': line number out of range"),
 	   program_name, p->lines_required);
@@ -782,7 +807,7 @@ handle_line_error (p, repetition)
   else
     fprintf (stderr, "\n");
 
-  cleanup ();
+  cleanup_fatal ();
 }
 
 /* Determine the line number that marks the end of this file,
@@ -791,9 +816,7 @@ handle_line_error (p, repetition)
    REPETITION is the repetition number. */
 
 static void
-process_line_count (p, repetition)
-     struct control *p;
-     int repetition;
+process_line_count (const struct control *p, int repetition)
 {
   unsigned int linenum;
   unsigned int last_line_to_save = p->lines_required * (repetition + 1);
@@ -825,10 +848,7 @@ process_line_count (p, repetition)
 }
 
 static void
-regexp_error (p, repetition, ignore)
-     struct control *p;
-     int repetition;
-     boolean ignore;
+regexp_error (struct control *p, int repetition, boolean ignore)
 {
   fprintf (stderr, _("%s: `%s': match not found"),
 	   program_name, global_argv[p->argnum]);
@@ -843,7 +863,7 @@ regexp_error (p, repetition, ignore)
       dump_rest_of_file ();
       close_output_file ();
     }
-  cleanup ();
+  cleanup_fatal ();
 }
 
 /* Read the input until a line matches the regexp in P, outputting
@@ -851,9 +871,7 @@ regexp_error (p, repetition, ignore)
    REPETITION is this repeat-count; 0 means the first time. */
 
 static void
-process_regexp (p, repetition)
-     struct control *p;
-     int repetition;
+process_regexp (struct control *p, int repetition)
 {
   struct cstring *line;		/* From input file. */
   unsigned int line_len;	/* To make "$" in regexps work. */
@@ -894,7 +912,7 @@ process_regexp (p, repetition)
 	  if (ret == -2)
 	    {
 	      error (0, 0, _("error in regular expression search"));
-	      cleanup ();
+	      cleanup_fatal ();
 	    }
 	  if (ret == -1)
 	    {
@@ -934,7 +952,7 @@ process_regexp (p, repetition)
 	  if (ret == -2)
 	    {
 	      error (0, 0, _("error in regular expression search"));
-	      cleanup ();
+	      cleanup_fatal ();
 	    }
 	  if (ret >= 0)
 	    break;
@@ -955,7 +973,7 @@ process_regexp (p, repetition)
 /* Split the input file according to the control records we have built. */
 
 static void
-split_file ()
+split_file (void)
 {
   unsigned int i, j;
 
@@ -983,8 +1001,7 @@ split_file ()
 /* Return the name of output file number NUM. */
 
 static char *
-make_filename (num)
-     unsigned int num;
+make_filename (unsigned int num)
 {
   strcpy (filename_space, prefix);
   if (suffix)
@@ -997,14 +1014,14 @@ make_filename (num)
 /* Create the next output file. */
 
 static void
-create_output_file ()
+create_output_file (void)
 {
   output_filename = make_filename (files_created);
   output_stream = fopen (output_filename, "w");
   if (output_stream == NULL)
     {
       error (0, errno, "%s", output_filename);
-      cleanup ();
+      cleanup_fatal ();
     }
   files_created++;
   bytes_written = 0;
@@ -1013,7 +1030,7 @@ create_output_file ()
 /* Delete all the files we have created. */
 
 static void
-delete_all_files ()
+delete_all_files (void)
 {
   unsigned int i;
   char *name;
@@ -1030,14 +1047,15 @@ delete_all_files ()
    of characters in this file. */
 
 static void
-close_output_file ()
+close_output_file (void)
 {
   if (output_stream)
     {
       if (fclose (output_stream) == EOF)
 	{
 	  error (0, errno, _("write error for `%s'"), output_filename);
-	  cleanup ();
+	  output_stream = NULL;
+	  cleanup_fatal ();
 	}
       if (bytes_written == 0 && elide_empty_files)
 	{
@@ -1052,27 +1070,11 @@ close_output_file ()
     }
 }
 
-/* Optionally remove files created so far; then exit.
-   Called when an error detected. */
-
-static void
-cleanup ()
-{
-  if (output_stream)
-    close_output_file ();
-
-  if (remove_files)
-    delete_all_files ();
-
-  exit (1);
-}
-
 /* Save line LINE to the output file and
    increment the character count for the current file. */
 
 static void
-save_line_to_file (line)
-     struct cstring *line;
+save_line_to_file (const struct cstring *line)
 {
   fwrite (line->str, sizeof (char), line->len, output_stream);
   bytes_written += line->len;
@@ -1081,7 +1083,7 @@ save_line_to_file (line)
 /* Return a new, initialized control record. */
 
 static struct control *
-new_control_record ()
+new_control_record (void)
 {
   static unsigned control_allocated = 0; /* Total space allocated. */
   struct control *p;
@@ -1096,7 +1098,8 @@ new_control_record ()
     {
       control_allocated += ALLOC_SIZE;
       controls = (struct control *)
-	xrealloc (controls, sizeof (struct control) * control_allocated);
+	xrealloc ((char *) controls,
+		  sizeof (struct control) * control_allocated);
     }
   p = &controls[control_used++];
   p->regexpr = NULL;
@@ -1107,49 +1110,23 @@ new_control_record ()
   return p;
 }
 
-/* Convert string NUM to an integer and put the value in *RESULT.
-   Return a TRUE if the string consists entirely of digits,
-   FALSE if not. */
-/* FIXME: use xstrtoul in place of this function.  */
-
-static boolean
-string_to_number (result, num)
-     int *result;
-     char *num;
-{
-  char ch;
-  int val = 0;
-
-  if (*num == '\0')
-    return FALSE;
-
-  while ((ch = *num++))
-    {
-      if (!ISDIGIT (ch))
-	return FALSE;
-      val = val * 10 + ch - '0';
-    }
-
-  *result = val;
-  return TRUE;
-}
-
 /* Check if there is a numeric offset after a regular expression.
    STR is the entire command line argument.
    P is the control record for this regular expression.
    NUM is the numeric part of STR. */
 
 static void
-check_for_offset (p, str, num)
-     struct control *p;
-     char *str;
-     char *num;
+check_for_offset (struct control *p, const char *str, const char *num)
 {
+  unsigned long val;
+
   if (*num != '-' && *num != '+')
     error (1, 0, _("%s: `+' or `-' expected after delimeter"), str);
 
-  if (!string_to_number (&p->offset, num + 1))
+  if (xstrtoul (num + 1, NULL, 10, &val, NULL) != LONGINT_OK
+      || val > UINT_MAX)
     error (1, 0, _("%s: integer expected after `%c'"), str, *num);
+  p->offset = (unsigned int) val;
 
   if (*num == '-')
     p->offset = -p->offset;
@@ -1161,11 +1138,9 @@ check_for_offset (p, str, num)
    ARGNUM is the ARGV index of STR. */
 
 static void
-parse_repeat_count (argnum, p, str)
-     int argnum;
-     struct control *p;
-     char *str;
+parse_repeat_count (int argnum, struct control *p, char *str)
 {
+  unsigned long val;
   char *end;
 
   end = str + strlen (str) - 1;
@@ -1176,9 +1151,15 @@ parse_repeat_count (argnum, p, str)
   if (str+1 == end-1 && *(str+1) == '*')
     p->repeat_forever = 1;
   else
-    if (!string_to_number (&p->repeat, str +  1))
-      error (1, 0, _("%s}: integer required between `{' and `}'"),
-	     global_argv[argnum]);
+    {
+      if (xstrtoul (str + 1, NULL, 10, &val, NULL) != LONGINT_OK
+	  || val > UINT_MAX)
+	{
+	  error (1, 0, _("%s}: integer required between `{' and `}'"),
+		 global_argv[argnum]);
+	}
+      p->repeat = (unsigned int) val;
+    }
 
   *end = '}';
 }
@@ -1190,10 +1171,7 @@ parse_repeat_count (argnum, p, str)
    Unless IGNORE is TRUE, mark these lines for output. */
 
 static struct control *
-extract_regexp (argnum, ignore, str)
-     int argnum;
-     boolean ignore;
-     char *str;
+extract_regexp (int argnum, boolean ignore, char *str)
 {
   int len;			/* Number of chars in this regexp. */
   char delim = *str;
@@ -1216,11 +1194,14 @@ extract_regexp (argnum, ignore, str)
   p->re_compiled.buffer = (unsigned char *) xmalloc (p->re_compiled.allocated);
   p->re_compiled.fastmap = xmalloc (256);
   p->re_compiled.translate = 0;
+#if !WITH_REGEX
+  p->re_compiled.syntax_parens = 0;
+#endif
   err = re_compile_pattern (p->regexpr, len, &p->re_compiled);
   if (err)
     {
       error (0, 0, _("%s: invalid regular expression: %s"), str, err);
-      cleanup ();
+      cleanup_fatal ();
     }
 
   if (closing_delim[1])
@@ -1233,13 +1214,11 @@ extract_regexp (argnum, ignore, str)
    After each pattern, check if the next argument is a repeat count. */
 
 static void
-parse_patterns (argc, start, argv)
-     int argc;
-     int start;
-     char **argv;
+parse_patterns (int argc, int start, char **argv)
 {
   int i;			/* Index into ARGV. */
   struct control *p;		/* New control record created. */
+  unsigned long val;
 
   for (i = start; i < argc; i++)
     {
@@ -1251,8 +1230,10 @@ parse_patterns (argc, start, argv)
 	{
 	  p = new_control_record ();
 	  p->argnum = i;
-	  if (!string_to_number (&p->lines_required, argv[i]))
+	  if (xstrtoul (argv[i], NULL, 10, &val, NULL) != LONGINT_OK
+	      || val > INT_MAX)
 	    error (1, 0, _("%s: invalid pattern"), argv[i]);
+	  p->lines_required = (int) val;
 	}
 
       if (i + 1 < argc && *argv[i + 1] == '{')
@@ -1265,8 +1246,7 @@ parse_patterns (argc, start, argv)
 }
 
 static unsigned
-get_format_flags (format_ptr)
-     char **format_ptr;
+get_format_flags (char **format_ptr)
 {
   unsigned count = 0;
 
@@ -1294,8 +1274,7 @@ get_format_flags (format_ptr)
 }
 
 static unsigned
-get_format_width (format_ptr)
-     char **format_ptr;
+get_format_width (char **format_ptr)
 {
   unsigned count = 0;
   char *start;
@@ -1318,8 +1297,7 @@ get_format_width (format_ptr)
 }
 
 static unsigned
-get_format_prec (format_ptr)
-     char **format_ptr;
+get_format_prec (char **format_ptr)
 {
   unsigned count = 0;
   char *start;
@@ -1359,8 +1337,7 @@ get_format_prec (format_ptr)
 }
 
 static void
-get_format_conv_type (format_ptr)
-     char **format_ptr;
+get_format_conv_type (char **format_ptr)
 {
   int ch = *((*format_ptr)++);
 
@@ -1387,8 +1364,7 @@ get_format_conv_type (format_ptr)
 }
 
 static unsigned
-max_out (format)
-     char *format;
+max_out (char *format)
 {
   unsigned out_count = 0;
   unsigned percents = 0;
@@ -1421,23 +1397,14 @@ max_out (format)
   return out_count;
 }
 
-static void
-interrupt_handler (signum)
-     int signum;
-{
-  error (0, 0, _("interrupted"));
-  cleanup ();
-}
-
 void
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   int optc;
-#ifdef _POSIX_VERSION
+  unsigned long val;
+#ifdef SA_INTERRUPT
   struct sigaction oldact, newact;
-#endif				/* _POSIX_VERSION */
+#endif				/* SA_INTERRUPT */
 
   program_name = argv[0];
   global_argv = argv;
@@ -1447,7 +1414,7 @@ main (argc, argv)
   remove_files = TRUE;
   prefix = DEFAULT_PREFIX;
 
-#ifdef _POSIX_VERSION
+#ifdef SA_INTERRUPT
   newact.sa_handler = interrupt_handler;
   sigemptyset (&newact.sa_mask);
   newact.sa_flags = 0;
@@ -1467,7 +1434,7 @@ main (argc, argv)
   sigaction (SIGTERM, NULL, &oldact);
   if (oldact.sa_handler != SIG_IGN)
     sigaction (SIGTERM, &newact, NULL);
-#else				/* !_POSIX_VERSION */
+#else
   if (signal (SIGHUP, SIG_IGN) != SIG_IGN)
     signal (SIGHUP, interrupt_handler);
   if (signal (SIGINT, SIG_IGN) != SIG_IGN)
@@ -1498,8 +1465,10 @@ main (argc, argv)
 	break;
 
       case 'n':
-	if (!string_to_number (&digits, optarg))
+	if (xstrtoul (optarg, NULL, 10, &val, NULL) != LONGINT_OK
+	    || val > INT_MAX)
 	  error (1, 0, _("%s: invalid number"), optarg);
+	digits = (int) val;
 	break;
 
       case 's':
@@ -1544,15 +1513,14 @@ main (argc, argv)
   if (close (input_desc) < 0)
     {
       error (0, errno, _("read error"));
-      cleanup ();
+      cleanup_fatal ();
     }
 
   exit (0);
 }
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
