@@ -25,8 +25,22 @@
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
+
+#if HAVE_LIMITS_H
+# include <limits.h>
+#endif
+
+#ifndef UINT_MAX
+# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
+#endif
+
+#ifndef INT_MAX
+# define INT_MAX ((int) (UINT_MAX >> 1))
+#endif
+
 #include "system.h"
 #include "version.h"
+#include "xstrtol.h"
 #include "error.h"
 
 char *xrealloc ();
@@ -35,9 +49,6 @@ char *xmalloc ();
 /* The name this program was run with. */
 char *program_name;
 
-static int adjust_column ();
-static int fold_file ();
-
 /* If nonzero, try to break on whitespace. */
 static int break_spaces;
 
@@ -47,10 +58,10 @@ static int count_bytes;
 /* If nonzero, at least one of the files we read was standard input. */
 static int have_read_stdin;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output then exit.  */
+/* If nonzero, print the version on standard output then exit.  */
 static int show_version;
 
 static struct option const longopts[] =
@@ -62,10 +73,9 @@ static struct option const longopts[] =
   {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
-
+
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
@@ -81,17 +91,163 @@ Wrap input lines in each FILE (standard input by default), writing to\n\
 standard output.\n\
 \n\
   -b, --bytes         count bytes rather than columns\n\
-  -s, --spaces        break at word boundaries\n\
+  -s, --spaces        break at spaces\n\
   -w, --width=WIDTH   use WIDTH columns instead of 80\n\
 "));
     }
   exit (status);
 }
 
+/* Assuming the current column is COLUMN, return the column that
+   printing C will move the cursor to.
+   The first column is 0. */
+
+static int
+adjust_column (int column, char c)
+{
+  if (!count_bytes)
+    {
+      if (c == '\b')
+	{
+	  if (column > 0)
+	    column--;
+	}
+      else if (c == '\r')
+	column = 0;
+      else if (c == '\t')
+	column = column + 8 - column % 8;
+      else /* if (isprint (c)) */
+	column++;
+    }
+  else
+    column++;
+  return column;
+}
+
+/* Fold file FILENAME, or standard input if FILENAME is "-",
+   to stdout, with maximum line length WIDTH.
+   Return 0 if successful, 1 if an error occurs. */
+
+static int
+fold_file (char *filename, int width)
+{
+  FILE *istream;
+  register int c;
+  int column = 0;		/* Screen column where next char will go. */
+  int offset_out = 0;	/* Index in `line_out' for next char. */
+  static char *line_out = NULL;
+  static int allocated_out = 0;
+
+  if (!strcmp (filename, "-"))
+    {
+      istream = stdin;
+      have_read_stdin = 1;
+    }
+  else
+    istream = fopen (filename, "r");
+
+  if (istream == NULL)
+    {
+      error (0, errno, "%s", filename);
+      return 1;
+    }
+
+  while ((c = getc (istream)) != EOF)
+    {
+      if (offset_out + 1 >= allocated_out)
+	{
+	  allocated_out += 1024;
+	  line_out = xrealloc (line_out, allocated_out);
+	}
+
+      if (c == '\n')
+	{
+	  line_out[offset_out++] = c;
+	  fwrite (line_out, sizeof (char), (size_t) offset_out, stdout);
+	  column = offset_out = 0;
+	  continue;
+	}
+
+    rescan:
+      column = adjust_column (column, c);
+
+      if (column > width)
+	{
+	  /* This character would make the line too long.
+	     Print the line plus a newline, and make this character
+	     start the next line. */
+	  if (break_spaces)
+	    {
+	      /* Look for the last blank. */
+	      int logical_end;
+
+	      for (logical_end = offset_out - 1; logical_end >= 0;
+		   logical_end--)
+		if (ISBLANK (line_out[logical_end]))
+		  break;
+	      if (logical_end >= 0)
+		{
+		  int i;
+
+		  /* Found a blank.  Don't output the part after it. */
+		  logical_end++;
+		  fwrite (line_out, sizeof (char), (size_t) logical_end,
+			  stdout);
+		  putchar ('\n');
+		  /* Move the remainder to the beginning of the next line.
+		     The areas being copied here might overlap. */
+		  memmove (line_out, line_out + logical_end,
+			   offset_out - logical_end);
+		  offset_out -= logical_end;
+		  for (column = i = 0; i < offset_out; i++)
+		    column = adjust_column (column, line_out[i]);
+		  goto rescan;
+		}
+	    }
+	  else
+	    {
+	      if (offset_out == 0)
+		{
+		  line_out[offset_out++] = c;
+		  continue;
+		}
+	    }
+	  line_out[offset_out++] = '\n';
+	  fwrite (line_out, sizeof (char), (size_t) offset_out, stdout);
+	  column = offset_out = 0;
+	  goto rescan;
+	}
+
+      line_out[offset_out++] = c;
+    }
+
+  if (offset_out)
+    fwrite (line_out, sizeof (char), (size_t) offset_out, stdout);
+
+  if (ferror (istream))
+    {
+      error (0, errno, "%s", filename);
+      if (strcmp (filename, "-"))
+	fclose (istream);
+      return 1;
+    }
+  if (strcmp (filename, "-") && fclose (istream) == EOF)
+    {
+      error (0, errno, "%s", filename);
+      return 1;
+    }
+
+  if (ferror (stdout))
+    {
+      error (0, errno, _("write error"));
+      return 1;
+    }
+
+  return 0;
+}
+
 void
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   int width = 80;
   int i;
@@ -133,9 +289,13 @@ main (argc, argv)
 	  break;
 
 	case 'w':		/* Line width. */
-	  width = atoi (optarg);
-	  if (width < 1)
-	    error (1, 0, _("%s: invalid line width"), optarg);
+	  {
+	    long int tmp_long;
+	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
+		|| tmp_long <= 0 || tmp_long > INT_MAX)
+	      error (1, 0, _("invalid number of columns: `%s'"), optarg);
+	    width = (int) tmp_long;
+	  }
 	  break;
 
 	default:
@@ -165,154 +325,3 @@ main (argc, argv)
 
   exit (errs);
 }
-
-/* Fold file FILENAME, or standard input if FILENAME is "-",
-   to stdout, with maximum line length WIDTH.
-   Return 0 if successful, 1 if an error occurs. */
-
-static int
-fold_file (filename, width)
-     char *filename;
-     int width;
-{
-  FILE *istream;
-  register int c;
-  int column = 0;		/* Screen column where next char will go. */
-  size_t offset_out = 0;	/* Index in `line_out' for next char. */
-  static char *line_out = NULL;
-  static size_t allocated_out = 0;
-
-  if (!strcmp (filename, "-"))
-    {
-      istream = stdin;
-      have_read_stdin = 1;
-    }
-  else
-    istream = fopen (filename, "r");
-
-  if (istream == NULL)
-    {
-      error (0, errno, "%s", filename);
-      return 1;
-    }
-
-  while ((c = getc (istream)) != EOF)
-    {
-      if (offset_out + 1 >= allocated_out)
-	{
-	  allocated_out += 1024;
-	  line_out = xrealloc (line_out, allocated_out);
-	}
-
-      if (c == '\n')
-	{
-	  line_out[offset_out++] = c;
-	  fwrite (line_out, sizeof (char), offset_out, stdout);
-	  column = offset_out = 0;
-	  continue;
-	}
-
-    rescan:
-      column = adjust_column (column, c);
-
-      if (column > width)
-	{
-	  /* This character would make the line too long.
-	     Print the line plus a newline, and make this character
-	     start the next line. */
-	  if (break_spaces)
-	    {
-	      /* Look for the last blank. */
-	      int logical_end;
-
-	      for (logical_end = offset_out - 1; logical_end >= 0;
-		   logical_end--)
-		if (ISBLANK (line_out[logical_end]))
-		  break;
-	      if (logical_end >= 0)
-		{
-		  int i;
-
-		  /* Found a blank.  Don't output the part after it. */
-		  logical_end++;
-		  fwrite (line_out, sizeof (char), logical_end, stdout);
-		  putchar ('\n');
-		  /* Move the remainder to the beginning of the next line.
-		     The areas being copied here might overlap. */
-		  memmove (line_out, line_out + logical_end,
-			   offset_out - logical_end);
-		  offset_out -= logical_end;
-		  for (column = i = 0; i < offset_out; i++)
-		    column = adjust_column (column, line_out[i]);
-		  goto rescan;
-		}
-	    }
-	  else
-	    {
-	      if (offset_out == 0)
-		{
-		  line_out[offset_out++] = c;
-		  continue;
-		}
-	    }
-	  line_out[offset_out++] = '\n';
-	  fwrite (line_out, sizeof (char), offset_out, stdout);
-	  column = offset_out = 0;
-	  goto rescan;
-	}
-
-      line_out[offset_out++] = c;
-    }
-
-  if (offset_out)
-    fwrite (line_out, sizeof (char), offset_out, stdout);
-
-  if (ferror (istream))
-    {
-      error (0, errno, "%s", filename);
-      if (strcmp (filename, "-"))
-	fclose (istream);
-      return 1;
-    }
-  if (strcmp (filename, "-") && fclose (istream) == EOF)
-    {
-      error (0, errno, "%s", filename);
-      return 1;
-    }
-
-  if (ferror (stdout))
-    {
-      error (0, errno, _("write error"));
-      return 1;
-    }
-
-  return 0;
-}
-
-/* Assuming the current column is COLUMN, return the column that
-   printing C will move the cursor to.
-   The first column is 0. */
-
-static int
-adjust_column (column, c)
-     int column;
-     char c;
-{
-  if (!count_bytes)
-    {
-      if (c == '\b')
-	{
-	  if (column > 0)
-	    column--;
-	}
-      else if (c == '\r')
-	column = 0;
-      else if (c == '\t')
-	column = column + 8 - column % 8;
-      else /* if (isprint (c)) */
-	column++;
-    }
-  else
-    column++;
-  return column;
-}
