@@ -15,26 +15,55 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   Written by Mike Haertel, mike@gnu.ai.mit.edu. */
+   Written by Mike Haertel, mike@gnu.ai.mit.edu.  */
 
 #include <config.h>
 
+#ifdef __GNUC__
+#define alloca __builtin_alloca
+#else /* not __GNUC__ */
+#if HAVE_ALLOCA_H
+#include <alloca.h>
+#else /* not HAVE_ALLOCA_H */
+#ifdef _AIX
+ #pragma alloca
+#else /* not _AIX */
+char *alloca ();
+#endif /* not _AIX */
+#endif /* not HAVE_ALLOCA_H */
+#endif /* not __GNUC__ */
+
 /* Get isblank from GNU libc.  */
 #define _GNU_SOURCE
 
 #include <stdio.h>
+#define NDEBUG
+#include <assert.h>
 #include <sys/types.h>
 #include <getopt.h>
+
+#if HAVE_LIMITS_H
+# include <limits.h>
+#endif
+
+#ifndef UINT_MAX
+# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
+#endif
+
+#ifndef INT_MAX
+# define INT_MAX ((int) (UINT_MAX >> 1))
+#endif
+
 #include "system.h"
 #include "version.h"
 #include "long-options.h"
+#include "xstrtol.h"
 #include "error.h"
 
 #define join system_join
 
 char *xmalloc ();
 char *xrealloc ();
-static void usage ();
 
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef min
@@ -42,69 +71,74 @@ static void usage ();
 #define min(A, B) ((A) < (B) ? (A) : (B))
 #define max(A, B) ((A) > (B) ? (A) : (B))
 
-/* An element of the list describing the format of each
-   output line. */
+/* An element of the list identifying which fields to print for each
+   output line.  */
 struct outlist
   {
-    int file;			/* File to take field from (1 or 2). */
-    int field;			/* Field number to print. */
+    /* File number: 0, 1, or 2.  0 means use the join field.
+       1 means use the first file argument, 2 the second.  */
+    int file;
+
+    /* Field index (zero-based), specified only when FILE is 1 or 2.  */
+    int field;
+
     struct outlist *next;
   };
 
-/* A field of a line. */
+/* A field of a line.  */
 struct field
   {
-    const char *beg;		/* First character in field. */
-    size_t len;			/* The length of the field. */
+    const char *beg;		/* First character in field.  */
+    size_t len;			/* The length of the field.  */
   };
 
-/* A line read from an input file.  Newlines are not stored. */
+/* A line read from an input file.  Newlines are not stored.  */
 struct line
   {
-    char *beg;			/* First character in line. */
-    char *lim;			/* Character after last character in line. */
-    int nfields;		/* Number of elements in `fields'. */
-    int nfields_allocated;	/* Number of elements in `fields'. */
+    char *beg;			/* First character in line.  */
+    char *lim;			/* Character after last character in line.  */
+    int nfields;		/* Number of elements in `fields'.  */
+    int nfields_allocated;	/* Number of elements in `fields'.  */
     struct field *fields;
   };
 
 /* One or more consecutive lines read from a file that all have the
-   same join field value. */
+   same join field value.  */
 struct seq
   {
-    int count;			/* Elements used in `lines'. */
-    int alloc;			/* Elements allocated in `lines'. */
+    int count;			/* Elements used in `lines'.  */
+    int alloc;			/* Elements allocated in `lines'.  */
     struct line *lines;
   };
 
-/* The name this program was run with. */
+/* The name this program was run with.  */
 char *program_name;
 
-/* If nonzero, print unpairable lines in file 1 or 2. */
+/* If nonzero, print unpairable lines in file 1 or 2.  */
 static int print_unpairables_1, print_unpairables_2;
 
-/* If nonzero, print pairable lines. */
+/* If nonzero, print pairable lines.  */
 static int print_pairables;
 
-/* Empty output field filler. */
+/* Empty output field filler.  */
 static char *empty_filler;
 
-/* Field to join on. */
+/* Field to join on.  */
 static int join_field_1, join_field_2;
 
-/* List of fields to print. */
-static struct outlist *outlist;
+/* List of fields to print.  */
+static struct outlist outlist_head;
 
-/* Last element in `outlist', where a new element can be added. */
-static struct outlist *outlist_end;
+/* Last element in `outlist', where a new element can be added.  */
+static struct outlist *outlist_end = &outlist_head;
 
 /* Tab character separating fields; if this is NUL fields are separated
    by any nonempty string of white space, otherwise by exactly one
-   tab character. */
+   tab character.  */
 static char tab;
 
 /* When using getopt_long_only, no long option can start with
-   a character that is a short option. */
+   a character that is a short option.  */
 static struct option const longopts[] =
 {
   {"j", required_argument, NULL, 'j'},
@@ -117,10 +151,48 @@ static struct option const longopts[] =
 static struct line uni_blank;
 
 static void
-ADD_FIELD (line, field, len)
-     struct line *line;
-     const char *field;
-     size_t len;
+usage (int status)
+{
+  if (status != 0)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
+  else
+    {
+      printf (_("\
+Usage: %s [OPTION]... FILE1 FILE2\n\
+"),
+	      program_name);
+      printf (_("\
+For each pair of input lines with identical join fields, write a line to\n\
+standard output.  The default join field is the first, delimited\n\
+by whitespace.  When FILE1 or FILE2 (not both) is -, read standard input.\n\
+\n\
+  -a SIDE          print unpairable lines coming from file SIDE\n\
+  -e EMPTY         replace missing input fields with EMPTY\n\
+  -j FIELD         (Obsolescent) equivalent to `-1 FIELD -2 FIELD'\n\
+  -j1 FIELD        (Obsolescent) equivalent to `-1 FIELD'\n\
+  -j2 FIELD        (Obsolescent) equivalent to `-2 FIELD'\n\
+  -1 FIELD         join on this FIELD of file 1\n\
+  -2 FIELD         join on this FIELD of file 2\n\
+  -o FORMAT        obey FORMAT while constructing output line\n\
+  -t CHAR          use CHAR as input and output field separator\n\
+  -v SIDE          like -a SIDE, but suppress joined output lines\n\
+  --help           display this help and exit\n\
+  --version        output version information and exit\n\
+\n\
+Unless -t CHAR is given, leading blanks separate fields and are ignored,\n\
+else fields are separated by CHAR.  Any FIELD is a field number counted\n\
+from 1.  FORMAT is one or more comma or blank separated specifications,\n\
+each being `SIDE.FIELD' or `0'.  Default FORMAT outputs the join field,\n\
+the remaining fields from FILE1, the remaining fields from FILE2, all\n\
+separated by CHAR.\n\
+"));
+    }
+  exit (status);
+}
+
+static void
+ADD_FIELD (struct line *line, const char *field, size_t len)
 {
   if (line->nfields >= line->nfields_allocated)
     {
@@ -134,11 +206,10 @@ ADD_FIELD (line, field, len)
   ++(line->nfields);
 }
 
-/* Fill in the `fields' structure in LINE. */
+/* Fill in the `fields' structure in LINE.  */
 
 static void
-xfields (line)
-     struct line *line;
+xfields (struct line *line)
 {
   int i;
   register char *ptr, *lim;
@@ -181,12 +252,10 @@ xfields (line)
 }
 
 /* Read a line from FP into LINE and split it into fields.
-   Return 0 if EOF, 1 otherwise. */
+   Return 0 if EOF, 1 otherwise.  */
 
 static int
-get_line (fp, line)
-     FILE *fp;
-     struct line *line;
+get_line (FILE *fp, struct line *line)
 {
   static int linesize = 80;
   int c, i;
@@ -223,8 +292,7 @@ get_line (fp, line)
 }
 
 static void
-freeline (line)
-     struct line *line;
+freeline (struct line *line)
 {
   free ((char *) line->fields);
   free (line->beg);
@@ -232,20 +300,17 @@ freeline (line)
 }
 
 static void
-initseq (seq)
-     struct seq *seq;
+initseq (struct seq *seq)
 {
   seq->count = 0;
   seq->alloc = 1;
   seq->lines = (struct line *) xmalloc (seq->alloc * sizeof (struct line));
 }
 
-/* Read a line from FP and add it to SEQ.  Return 0 if EOF, 1 otherwise. */
+/* Read a line from FP and add it to SEQ.  Return 0 if EOF, 1 otherwise.  */
 
 static int
-getseq (fp, seq)
-     FILE *fp;
-     struct seq *seq;
+getseq (FILE *fp, struct seq *seq)
 {
   if (seq->count == seq->alloc)
     {
@@ -263,8 +328,7 @@ getseq (fp, seq)
 }
 
 static void
-delseq (seq)
-     struct seq *seq;
+delseq (struct seq *seq)
 {
   int i;
   for (i = 0; i < seq->count; i++)
@@ -274,15 +338,13 @@ delseq (seq)
 }
 
 /* Return <0 if the join field in LINE1 compares less than the one in LINE2;
-   >0 if it compares greater; 0 if it compares equal. */
+   >0 if it compares greater; 0 if it compares equal.  */
 
 static int
-keycmp (line1, line2)
-     struct line *line1;
-     struct line *line2;
+keycmp (struct line *line1, struct line *line2)
 {
-  const char *beg1, *beg2;	/* Start of field to compare in each file. */
-  int len1, len2;		/* Length of fields to compare. */
+  const char *beg1, *beg2;	/* Start of field to compare in each file.  */
+  int len1, len2;		/* Length of fields to compare.  */
   int diff;
 
   if (join_field_1 < line1->nfields)
@@ -318,12 +380,10 @@ keycmp (line1, line2)
 }
 
 /* Print field N of LINE if it exists and is nonempty, otherwise
-   `empty_filler' if it is nonempty. */
+   `empty_filler' if it is nonempty.  */
 
 static void
-prfield (n, line)
-     int n;
-     struct line *line;
+prfield (int n, struct line *line)
 {
   int len;
 
@@ -339,22 +399,47 @@ prfield (n, line)
     fputs (empty_filler, stdout);
 }
 
-/* Print the join of LINE1 and LINE2. */
+/* Print the join of LINE1 and LINE2.  */
 
 static void
-prjoin (line1, line2)
-     struct line *line1;
-     struct line *line2;
+prjoin (struct line *line1, struct line *line2)
 {
+  const struct outlist *outlist;
+
+  outlist = outlist_head.next;
   if (outlist)
     {
-      struct outlist *o;
+      const struct outlist *o;
 
-      prfield (outlist->field - 1, outlist->file == 1 ? line1 : line2);
-      for (o = outlist->next; o; o = o->next)
+      o = outlist;
+      while (1)
 	{
+	  int field;
+	  struct line *line;
+
+	  if (o->file == 0)
+	    {
+	      if (line1 == &uni_blank)
+	        {
+		  line = line2;
+		  field = join_field_2;
+		}
+	      else
+	        {
+		  line = line1;
+		  field = join_field_1;
+		}
+	    }
+	  else
+	    {
+	      line = (o->file == 1 ? line1 : line2);
+	      field = o->field;
+	    }
+	  prfield (field, line);
+	  o = o->next;
+	  if (o == NULL)
+	    break;
 	  putchar (tab ? tab : ' ');
-	  prfield (o->field - 1, o->file == 1 ? line1 : line2);
 	}
       putchar ('\n');
     }
@@ -395,18 +480,16 @@ prjoin (line1, line2)
     }
 }
 
-/* Print the join of the files in FP1 and FP2. */
+/* Print the join of the files in FP1 and FP2.  */
 
 static void
-join (fp1, fp2)
-     FILE *fp1;
-     FILE *fp2;
+join (FILE *fp1, FILE *fp2)
 {
   struct seq seq1, seq2;
   struct line line;
   int diff, i, j, eof1, eof2;
 
-  /* Read the first line of each file. */
+  /* Read the first line of each file.  */
   initseq (&seq1);
   getseq (fp1, &seq1);
   initseq (&seq2);
@@ -435,7 +518,7 @@ join (fp1, fp2)
 	}
 
       /* Keep reading lines from file1 as long as they continue to
-         match the current line from file2. */
+         match the current line from file2.  */
       eof1 = 0;
       do
 	if (!getseq (fp1, &seq1))
@@ -447,7 +530,7 @@ join (fp1, fp2)
       while (!keycmp (&seq1.lines[seq1.count - 1], &seq2.lines[0]));
 
       /* Keep reading lines from file2 as long as they continue to
-         match the current line from file1. */
+         match the current line from file1.  */
       eof2 = 0;
       do
 	if (!getseq (fp2, &seq2))
@@ -512,85 +595,118 @@ join (fp1, fp2)
   delseq (&seq2);
 }
 
-/* Add a field spec for field FIELD of file FILE to `outlist' and return 1,
-   unless either argument is invalid; then just return 0. */
+/* Add a field spec for field FIELD of file FILE to `outlist'.  */
 
-static int
-add_field (file, field)
-     int file;
-     int field;
+static void
+add_field (int file, int field)
 {
   struct outlist *o;
 
-  if (file < 1 || file > 2 || field < 1)
-    return 0;
+  assert (file == 0 || file == 1 || file == 2);
+  assert (field >= 0);
+
   o = (struct outlist *) xmalloc (sizeof (struct outlist));
   o->file = file;
   o->field = field;
   o->next = NULL;
 
-  /* Add to the end of the list so the fields are in the right order. */
-  if (outlist == NULL)
-    outlist = o;
-  else
-    outlist_end->next = o;
+  /* Add to the end of the list so the fields are in the right order.  */
+  outlist_end->next = o;
   outlist_end = o;
-
-  return 1;
 }
 
-/* Add the comma or blank separated field spec(s) in STR to `outlist'.
-   Return the number of fields added. */
+/* Convert a single field specifier string, S, to a *FILE_INDEX, *FIELD_INDEX
+   pair.  In S, the field index string is 1-based; *FIELD_INDEX is zero-based.
+   If S is valid, return zero.  Otherwise, give a diagnostic, don't update
+   *FILE_INDEX or *FIELD_INDEX, and return nonzero.  */
 
 static int
-add_field_list (str)
-     char *str;
+decode_field_spec (const char *s, int *file_index, int *field_index)
 {
-  int added = 0;
-  int file = -1, field = -1;
-  int dot_found = 0;
+  int valid = 0;
 
-  for (; *str; str++)
+  /* The first character must be 0, 1, or 2.  */
+  switch (s[0])
     {
-      if (*str == ',' || ISBLANK (*str))
+    case '0':
+      if (s[1] == '\0')
 	{
-	  added += add_field (file, field);
-	  uni_blank.nfields = max (uni_blank.nfields, field);
-	  file = field = -1;
-	  dot_found = 0;
+	  *file_index = 0;
+	  /* Leave *field_index undefined.  */
+	  valid = 1;
 	}
-      else if (*str == '.')
-	dot_found = 1;
-      else if (ISDIGIT (*str))
-	{
-	  if (!dot_found)
+      else
+        {
+	  /* `0' must be all alone -- no `.FIELD'.  */
+	  error (0, 0, _("invalid field specifier: `%s'"), s);
+	}
+      break;
+
+    case '1':
+    case '2':
+      if (s[1] == '.' && s[2] != '\0')
+        {
+	  strtol_error s_err;
+	  long int tmp_long;
+
+	  s_err = xstrtol (s + 2, NULL, 10, &tmp_long, NULL);
+	  if (s_err != LONGINT_OK || tmp_long <= 0 || tmp_long > INT_MAX)
 	    {
-	      if (file == -1)
-		file = 0;
-	      file = file * 10 + *str - '0';
+	      error (0, 0, _("invalid field number: `%s'"), s + 2);
 	    }
 	  else
 	    {
-	      if (field == -1)
-		field = 0;
-	      field = field * 10 + *str - '0';
+	      *file_index = s[0] - '0';
+	      /* Convert to a zero-based index.  */
+	      *field_index = (int) tmp_long - 1;
+	      valid = 1;
 	    }
 	}
-      else
-	return 0;
-    }
+      break;
 
-  uni_blank.nfields = max (uni_blank.nfields, field);
-  added += add_field (file, field);
-  return added;
+    default:
+      error (0, 0, _("invalid file number in field spec: `%s'"), s);
+      break;
+    }
+  return !valid;
 }
 
-/* Create a blank line with COUNT fields separated by tabs. */
+/* Add the comma or blank separated field spec(s) in STR to `outlist'.
+   Return nonzero to indicate failure.  */
+
+static int
+add_field_list (const char *c_str)
+{
+  char *p, *str;
+
+  /* Make a writable copy of c_str.  */
+  str = (char *) alloca (strlen (c_str) + 1);
+  strcpy (str, c_str);
+
+  p = str;
+  do
+    {
+      int invalid;
+      int file_index, field_index;
+      char *spec_item = p;
+
+      p = strpbrk (p, ", \t");
+      if (p)
+        *p++ = 0;
+      invalid = decode_field_spec (spec_item, &file_index, &field_index);
+      if (invalid)
+	return 1;
+      add_field (file_index, field_index);
+      uni_blank.nfields = max (uni_blank.nfields, field_index);
+    }
+  while (p);
+  return 0;
+}
+
+/* Create a blank line with COUNT fields separated by tabs.  */
 
 void
-make_blank (blank, count)
-     struct line *blank;
-     int count;
+make_blank (struct line *blank, int count)
 {
   int i;
   blank->nfields = count;
@@ -607,13 +723,11 @@ make_blank (blank, count)
 }
 
 void
-main (argc, argv)
-     int argc;
-     char *argv[];
+main (int argc, char **argv)
 {
   char *names[2];
   FILE *fp1, *fp2;
-  int optc, prev_optc = 0, nfiles, val;
+  int optc, prev_optc = 0, nfiles;
 
   program_name = argv[0];
 
@@ -629,19 +743,25 @@ main (argc, argv)
   while ((optc = getopt_long_only (argc, argv, "-a:e:1:2:o:t:v:", longopts,
 				   (int *) 0)) != EOF)
     {
+      long int val;
+
       switch (optc)
 	{
 	case 0:
 	  break;
 
+	case 'v':
+	    print_pairables = 0;
+	    /* Fall through.  */
+
 	case 'a':
-	  val = atoi (optarg);
+	  if (xstrtol (optarg, NULL, 10, &val, NULL) != LONGINT_OK
+	      || (val != 1 && val != 2))
+	    error (2, 0, _("invalid field number: `%s'"), optarg);
 	  if (val == 1)
 	    print_unpairables_1 = 1;
-	  else if (val == 2)
-	    print_unpairables_2 = 1;
 	  else
-	    error (2, 0, _("invalid file number for `-a'"));
+	    print_unpairables_2 = 1;
 	  break;
 
 	case 'e':
@@ -649,56 +769,52 @@ main (argc, argv)
 	  break;
 
 	case '1':
-	  val = atoi (optarg);
-	  if (val <= 0)
-	    error (2, 0, _("invalid field number for `-1'"));
-	  join_field_1 = val - 1;
+	  if (xstrtol (optarg, NULL, 10, &val, NULL) != LONGINT_OK
+	      || val <= 0 || val > INT_MAX)
+	    {
+	      error (2, 0, _("invalid field number for file 1: `%s'"), optarg);
+	    }
+	  join_field_1 = (int) val - 1;
 	  break;
 
 	case '2':
-	  val = atoi (optarg);
-	  if (val <= 0)
-	    error (2, 0, _("invalid field number for `-2'"));
-	  join_field_2 = val - 1;
+	  if (xstrtol (optarg, NULL, 10, &val, NULL) != LONGINT_OK
+	      || val <= 0 || val > INT_MAX)
+	    error (2, 0, _("invalid field number for file 2: `%s'"), optarg);
+	  join_field_2 = (int) val - 1;
 	  break;
 
 	case 'j':
-	  val = atoi (optarg);
-	  if (val <= 0)
-	    error (2, 0, _("invalid field number for `-j'"));
-	  join_field_1 = join_field_2 = val - 1;
+	  if (xstrtol (optarg, NULL, 10, &val, NULL) != LONGINT_OK
+	      || val <= 0 || val > INT_MAX)
+	    error (2, 0, _("invalid field number: `%s'"), optarg);
+	  join_field_1 = join_field_2 = (int) val - 1;
 	  break;
 
 	case 'o':
-	  if (add_field_list (optarg) == 0)
-	    error (2, 0, _("invalid field list for `-o'"));
+	  if (add_field_list (optarg))
+	    exit (1);
 	  break;
 
 	case 't':
 	  tab = *optarg;
 	  break;
 
-	case 'v':
-	  val = atoi (optarg);
-	  if (val == 1)
-	    print_unpairables_1 = 1;
-	  else if (val == 2)
-	    print_unpairables_2 = 1;
-	  else
-	    error (2, 0, _("invalid file number for `-v'"));
-	  print_pairables = 0;
-	  break;
-
-	case 1:		/* Non-option argument. */
-	  if (prev_optc == 'o')
+	case 1:		/* Non-option argument.  */
+	  if (prev_optc == 'o' && optind <= argc - 2)
 	    {
-	      /* Might be continuation of args to -o. */
-	      if (add_field_list (optarg) > 0)
-		continue;	/* Don't change `prev_optc'. */
+	      if (add_field_list (optarg))
+		exit (1);
+
+	      /* Might be continuation of args to -o.  */
+	      continue;		/* Don't change `prev_optc'.  */
 	    }
 
 	  if (nfiles > 1)
-	    usage (1);
+	    {
+	      error (0, 0, "too many non-option arguments");
+	      usage (1);
+	    }
 	  names[nfiles++] = optarg;
 	  break;
 
@@ -713,7 +829,10 @@ main (argc, argv)
   make_blank (&uni_blank, uni_blank.nfields);
 
   if (nfiles != 2)
-    usage (1);
+    {
+      error (0, 0, "too few non-option arguments");
+      usage (1);
+    }
 
   fp1 = strcmp (names[0], "-") ? fopen (names[0], "r") : stdin;
   if (!fp1)
@@ -725,6 +844,10 @@ main (argc, argv)
     error (1, errno, _("both files cannot be standard input"));
   join (fp1, fp2);
 
+  if (fp1 != stdin && fclose (fp1) == EOF)
+    error (1, errno, "%s", names[0]);
+  if (fp2 != stdin && fclose (fp2) == EOF)
+    error (1, errno, "%s", names[1]);
   if ((fp1 == stdin || fp2 == stdin) && fclose (stdin) == EOF)
     error (1, errno, "-");
   if (ferror (stdout) || fclose (stdout) == EOF)
@@ -732,42 +855,3 @@ main (argc, argv)
 
   exit (0);
 }
-
-static void
-usage (status)
-     int status;
-{
-  if (status != 0)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
-  else
-    {
-      printf (_("\
-Usage: %s [OPTION]... FILE1 FILE2\n\
-"),
-	      program_name);
-      printf (_("\
-For each pair of input lines with identical join fields, write a line to\n\
-standard output.  The default join field is the first, delimited\n\
-by whitespace.  When FILE1 or FILE2 (not both) is -, read standard input.\n\
-\n\
-  -a SIDE          print unpairable lines coming from file SIDE\n\
-  -e EMPTY         replace missing input fields with EMPTY\n\
-  -j FIELD         join on this FIELD for both files\n\
-  -[j]SIDE FIELD   join on this FIELD for file SIDE\n\
-  -o FORMAT        obey FORMAT while constructing output line\n\
-  -t CHAR          use CHAR as input and output field separator\n\
-  -v SIDE          like -a SIDE, but suppress joined output lines\n\
-  --help           display this help and exit\n\
-  --version        output version information and exit\n\
-\n\
-SIDE is 1 for FILE1 or 2 for FILE2.  Unless -t CHAR is given, leading blanks\n\
-separate fields and are ignored, else fields are separated by CHAR.\n\
-Any FIELD is a field number counted from 1.  FORMAT is one or more\n\
-comma or blank separated specifications, each being `SIDE.FIELD'.\n\
-Default FORMAT outputs the join field, the remaining fields from\n\
-FILE1, the remaining fields from FILE2, all separated by CHAR.\n\
-"));
-    }
-  exit (status);
-}
