@@ -116,8 +116,7 @@ The following two options are useful only when verifying checksums:\n\
 The sums are computed as described in RFC 1321.  When checking, the input\n\
 should be a former output of this program.  The default mode is to print\n\
 a line with checksum, a character indicating type (`*' for binary, ` ' for\n\
-text), and name for each FILE.  The --status and --warn options are\n\
-meaningful only when verifying checksums.\n"),
+text), and name for each FILE.\n"),
 	    program_name, program_name, program_name);
 
   exit (status);
@@ -166,8 +165,6 @@ split_3 (char *s, char **u, int *binary, char **w)
   return 1;
 }
 
-/* FIXME: use strcspn.  */
-
 static int
 hex_digits (const char *s)
 {
@@ -181,8 +178,6 @@ hex_digits (const char *s)
 }
 
 /* FIXME: allow newline in filename by encoding it. */
-/* FIXME: distinguish between file open/read failure and inconsistent
-   checksum. */
 
 static int
 md5_file (const char *filename, int binary, unsigned char *md5_result)
@@ -367,25 +362,18 @@ md5_check (const char *checkfile_name, int binary)
 
 	  if (n_open_or_read_failures > 0)
 	    {
-	      printf (n_properly_formated_lines == 1
-		? _("WARNING: the single listed file could not be read\n")
-		: _("WARNING: %d of the listed files could not be read\n"),
-		      n_open_or_read_failures);
+	      error (0, 0,
+		   _("WARNING: %d of %d listed file%s could not be read\n"),
+		     n_open_or_read_failures, n_properly_formated_lines,
+		     (n_properly_formated_lines == 1 ? "" : "s"));
 	    }
 
-	  if (n_mismatched_checksums == 0)
+	  if (n_mismatched_checksums > 0)
 	    {
-	      printf (n_properly_formated_lines == 1
-		      ? _("the single computed checksum matched\n")
-		      : _("all %d computed checksums matched\n"),
-		      n_computed_checkums);
-	    }
-	  else
-	    {
-	      printf (n_properly_formated_lines == 1
-	      ? _("WARNING: the single computed checksum did NOT match\n")
-	      : _("WARNING: %d out of %d computed checksums did NOT match\n"),
-		      n_mismatched_checksums, n_computed_checkums);
+	      error (0, 0,
+		   _("WARNING: %d of %d computed checksum%s did NOT match\n"),
+		     n_mismatched_checksums, n_computed_checkums,
+		     (n_computed_checkums == 1 ? "" : "s"));
 	    }
 	}
     }
@@ -402,7 +390,7 @@ main (int argc, char **argv)
   int do_version = 0;
   int opt;
   char **string = NULL;
-  char n_strings = 0;
+  size_t n_strings = 0;
   size_t i;
   size_t err = 0;
 
