@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Jim Meyering.  */
 
@@ -54,37 +54,37 @@ typedef double LONG_DOUBLE;
 #endif
 
 #if HAVE_LIMITS_H
-#include <limits.h>
+# include <limits.h>
 #endif
 #ifndef SCHAR_MAX
-#define SCHAR_MAX 127
+# define SCHAR_MAX 127
 #endif
 #ifndef SCHAR_MIN
-#define SCHAR_MIN (-128)
+# define SCHAR_MIN (-128)
 #endif
 #ifndef SHRT_MAX
-#define SHRT_MAX 32767
+# define SHRT_MAX 32767
 #endif
 #ifndef SHRT_MIN
-#define SHRT_MIN (-32768)
+# define SHRT_MIN (-32768)
 #endif
 #ifndef ULONG_MAX
-#define ULONG_MAX ((unsigned long) ~(unsigned long) 0)
+# define ULONG_MAX ((unsigned long) ~(unsigned long) 0)
 #endif
 #ifndef OFF_T_MAX
 /* FIXME: is there a way to do this without relying on the
    `8 bits per byte' assumption?  */
-#define OFF_T_MAX (~((off_t)1 << (sizeof (off_t) * 8 - 1)))
+# define OFF_T_MAX (~((off_t)1 << (sizeof (off_t) * 8 - 1)))
 #endif
 
 #define STREQ(a,b) (strcmp((a), (b)) == 0)
 
 #ifndef MAX
-#define MAX(a, b) ((a) > (b) ? (a) : (b))
+# define MAX(a, b) ((a) > (b) ? (a) : (b))
 #endif
 
 #ifndef MIN
-#define MIN(a,b) (((a) < (b)) ? (a) : (b))
+# define MIN(a,b) (((a) < (b)) ? (a) : (b))
 #endif
 
 /* The default number of input bytes per output line.  */
@@ -92,17 +92,17 @@ typedef double LONG_DOUBLE;
 
 /* The number of decimal digits of precision in a float.  */
 #ifndef FLT_DIG
-#define FLT_DIG 7
+# define FLT_DIG 7
 #endif
 
 /* The number of decimal digits of precision in a double.  */
 #ifndef DBL_DIG
-#define DBL_DIG 15
+# define DBL_DIG 15
 #endif
 
 /* The number of decimal digits of precision in a long double.  */
 #ifndef LDBL_DIG
-#define LDBL_DIG DBL_DIG
+# define LDBL_DIG DBL_DIG
 #endif
 
 char *xmalloc ();
@@ -217,20 +217,20 @@ static long int pseudo_offset;
 
 /* Function to format an address and optionally an additional parenthesized
    pseudo-address; it returns the formatted string.  */
-static const char *(*format_address) (/* long unsigned int */);
+static const char *(*format_address) __P ((long unsigned int));
 
 /* The number of input bytes to skip before formatting and writing.  */
 static off_t n_bytes_to_skip = 0;
 
-/* When non-zero, MAX_BYTES_TO_FORMAT is the maximum number of bytes
+/* When nonzero, MAX_BYTES_TO_FORMAT is the maximum number of bytes
    to be read and formatted.  Otherwise all input is formatted.  */
 static int limit_bytes_to_format = 0;
 
 /* The maximum number of bytes that will be formatted.  This
-   value is used only when LIMIT_BYTES_TO_FORMAT is non-zero.  */
+   value is used only when LIMIT_BYTES_TO_FORMAT is nonzero.  */
 static off_t max_bytes_to_format;
 
-/* When non-zero and two or more consecutive blocks are equal, format
+/* When nonzero and two or more consecutive blocks are equal, format
    only the first block and output an asterisk alone on the following
    line to indicate that identical blocks have been elided.  */
 static int abbreviate_duplicate_blocks = 1;
@@ -262,7 +262,7 @@ static char const *const *file_list;
 /* The input stream associated with the current file.  */
 static FILE *in_stream;
 
-/* If non-zero, at least one of the files we read was standard input.  */
+/* If nonzero, at least one of the files we read was standard input.  */
 static int have_read_stdin;
 
 #define LONGEST_INTEGRAL_TYPE long int
@@ -273,10 +273,10 @@ static enum size_spec integral_type_size[MAX_INTEGRAL_TYPE_SIZE + 1];
 #define MAX_FP_TYPE_SIZE sizeof(LONG_DOUBLE)
 static enum size_spec fp_type_size[MAX_FP_TYPE_SIZE + 1];
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output then exit.  */
+/* If nonzero, print the version on standard output then exit.  */
 static int show_version;
 
 static struct option const long_options[] =
@@ -298,8 +298,7 @@ static struct option const long_options[] =
 };
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
@@ -374,9 +373,7 @@ uses -A o -t d2 -w 16.\n\
    using Euclid's algorithm.  */
 
 static unsigned int
-gcd (u, v)
-     unsigned int u;
-     unsigned int v;
+gcd (unsigned int u, unsigned int v)
 {
   unsigned int t;
   while (v != 0)
@@ -391,9 +388,7 @@ gcd (u, v)
 /* Compute the least common multiple of U and V.  */
 
 static unsigned int
-lcm (u, v)
-     unsigned int u;
-     unsigned int v;
+lcm (unsigned int u, unsigned int v)
 {
   unsigned int t = gcd (u, v);
   if (t == 0)
@@ -402,10 +397,8 @@ lcm (u, v)
 }
 
 static void
-print_s_char (n_bytes, block, fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *fmt_string;
+print_s_char (long unsigned int n_bytes, const char *block,
+	      const char *fmt_string)
 {
   int i;
   for (i = n_bytes; i > 0; i--)
@@ -420,10 +413,8 @@ print_s_char (n_bytes, block, fmt_string)
 }
 
 static void
-print_char (n_bytes, block, fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *fmt_string;
+print_char (long unsigned int n_bytes, const char *block,
+	    const char *fmt_string)
 {
   int i;
   for (i = n_bytes; i > 0; i--)
@@ -435,10 +426,8 @@ print_char (n_bytes, block, fmt_string)
 }
 
 static void
-print_s_short (n_bytes, block, fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *fmt_string;
+print_s_short (long unsigned int n_bytes, const char *block,
+	       const char *fmt_string)
 {
   int i;
   for (i = n_bytes / sizeof (unsigned short); i > 0; i--)
@@ -452,10 +441,8 @@ print_s_short (n_bytes, block, fmt_string)
     }
 }
 static void
-print_short (n_bytes, block, fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *fmt_string;
+print_short (long unsigned int n_bytes, const char *block,
+	     const char *fmt_string)
 {
   int i;
   for (i = n_bytes / sizeof (unsigned short); i > 0; i--)
@@ -467,10 +454,8 @@ print_short (n_bytes, block, fmt_string)
 }
 
 static void
-print_int (n_bytes, block, fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *fmt_string;
+print_int (long unsigned int n_bytes, const char *block,
+	   const char *fmt_string)
 {
   int i;
   for (i = n_bytes / sizeof (unsigned int); i > 0; i--)
@@ -482,10 +467,8 @@ print_int (n_bytes, block, fmt_string)
 }
 
 static void
-print_long (n_bytes, block, fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *fmt_string;
+print_long (long unsigned int n_bytes, const char *block,
+	    const char *fmt_string)
 {
   int i;
   for (i = n_bytes / sizeof (unsigned long); i > 0; i--)
@@ -497,10 +480,8 @@ print_long (n_bytes, block, fmt_string)
 }
 
 static void
-print_float (n_bytes, block, fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *fmt_string;
+print_float (long unsigned int n_bytes, const char *block,
+	     const char *fmt_string)
 {
   int i;
   for (i = n_bytes / sizeof (float); i > 0; i--)
@@ -512,10 +493,8 @@ print_float (n_bytes, block, fmt_string)
 }
 
 static void
-print_double (n_bytes, block, fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *fmt_string;
+print_double (long unsigned int n_bytes, const char *block,
+	      const char *fmt_string)
 {
   int i;
   for (i = n_bytes / sizeof (double); i > 0; i--)
@@ -528,10 +507,8 @@ print_double (n_bytes, block, fmt_string)
 
 #ifdef HAVE_LONG_DOUBLE
 static void
-print_long_double (n_bytes, block, fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *fmt_string;
+print_long_double (long unsigned int n_bytes, const char *block,
+		   const char *fmt_string)
 {
   int i;
   for (i = n_bytes / sizeof (LONG_DOUBLE); i > 0; i--)
@@ -545,10 +522,8 @@ print_long_double (n_bytes, block, fmt_string)
 #endif
 
 static void
-print_named_ascii (n_bytes, block, unused_fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *unused_fmt_string;
+print_named_ascii (long unsigned int n_bytes, const char *block,
+		   const char *unused_fmt_string)
 {
   int i;
   for (i = n_bytes; i > 0; i--)
@@ -574,10 +549,8 @@ print_named_ascii (n_bytes, block, unused_fmt_string)
 }
 
 static void
-print_ascii (n_bytes, block, unused_fmt_string)
-     long unsigned int n_bytes;
-     const char *block;
-     const char *unused_fmt_string;
+print_ascii (long unsigned int n_bytes, const char *block,
+	     const char *unused_fmt_string)
 {
   int i;
   for (i = n_bytes; i > 0; i--)
@@ -634,15 +607,12 @@ print_ascii (n_bytes, block, unused_fmt_string)
    unsigned long integer value.  If S points to a non-digit set *P to S,
    *VAL to 0, and return 0.  Otherwise, accumulate the integer value of
    the string of digits.  If the string of digits represents a value
-   larger than ULONG_MAX, don't modify *VAL or *P and return non-zero.
+   larger than ULONG_MAX, don't modify *VAL or *P and return nonzero.
    Otherwise, advance *P to the first non-digit after S, set *VAL to
    the result of the conversion and return zero.  */
 
 static int
-simple_strtoul (s, p, val)
-     const char *s;
-     const char **p;
-     long unsigned int *val;
+simple_strtoul (const char *s, const char **p, long unsigned int *val)
 {
   unsigned long int sum;
 
@@ -663,7 +633,7 @@ simple_strtoul (s, p, val)
    description of that format in *TSPEC, make *NEXT point at the character
    following the just-decoded format (if *NEXT is non-NULL), and return
    zero.  If S is not valid, don't modify *NEXT or *TSPEC and return
-   non-zero.  For example, if S were "d4afL" *NEXT would be set to "afL"
+   nonzero.  For example, if S were "d4afL" *NEXT would be set to "afL"
    and *TSPEC would be
      {
        fmt = SIGNED_DECIMAL;
@@ -674,10 +644,7 @@ simple_strtoul (s, p, val)
    */
 
 static int
-decode_one_format (s, next, tspec)
-     const char *s;
-     const char **next;
-     struct tspec *tspec;
+decode_one_format (const char *s, const char **next, struct tspec *tspec)
 {
   enum size_spec size_spec;
   unsigned long int size;
@@ -906,11 +873,10 @@ decode_one_format (s, next, tspec)
 
 /* Decode the POSIX-style od format string S.  Append the decoded
    representation to the global array SPEC, reallocating SPEC if
-   necessary.  Return zero if S is valid, non-zero otherwise.  */
+   necessary.  Return zero if S is valid, nonzero otherwise.  */
 
 static int
-decode_format_string (s)
-     const char *s;
+decode_format_string (const char *s)
 {
   assert (s != NULL);
 
@@ -943,13 +909,12 @@ decode_format_string (s)
    file pointer IN_STREAM to position N_SKIP in the concatenation of
    those files.  If any file operation fails or if there are fewer than
    N_SKIP bytes in the combined input, give an error message and return
-   non-zero.  When possible, use seek- rather than read operations to
+   nonzero.  When possible, use seek- rather than read operations to
    advance IN_STREAM.  A file name of "-" is interpreted as standard
    input.  */
 
 static int
-skip (n_skip)
-     off_t n_skip;
+skip (off_t n_skip)
 {
   int err;
 
@@ -1016,7 +981,13 @@ skip (n_skip)
 	    }
 	  else
 	    {
-	      if (fseek (in_stream, n_skip, SEEK_SET) == 0)
+	      /* fseek may work on some streams for which lseek doesn't.
+		 But fseek's offset argument is restricted to the range
+		 of type `long'.  So if N_SKIP is too large or if fseek
+		 fails, try lseek.  */
+	      if ((n_skip <= LONG_MAX
+		   && fseek (in_stream, (long) n_skip, SEEK_SET) == 0)
+		  || lseek (fileno (in_stream), n_skip, SEEK_SET) >= 0)
 		{
 		  n_skip = 0;
 		  break;
@@ -1024,7 +995,8 @@ skip (n_skip)
 	    }
 	}
 
-      /* fseek didn't work or wasn't attempted; do it the slow way.  */
+      /* Seek didn't work or wasn't attempted;  position the file pointer
+	 by reading.  */
 
       for (j = n_skip / BUFSIZ; j >= 0; j--)
 	{
@@ -1050,15 +1022,13 @@ skip (n_skip)
 }
 
 static const char *
-format_address_none (address)
-     long unsigned int address;
+format_address_none (long unsigned int address)
 {
   return "";
 }
 
 static const char *
-format_address_std (address)
-     long unsigned int address;
+format_address_std (long unsigned int address)
 {
   const char *address_string;
 
@@ -1068,8 +1038,7 @@ format_address_std (address)
 }
 
 static const char *
-format_address_label (address)
-     long unsigned int address;
+format_address_label (long unsigned int address)
 {
   const char *address_string;
   assert (output_address_fmt_string != NULL);
@@ -1092,11 +1061,8 @@ format_address_label (address)
    only when it has not been padded to length BYTES_PER_BLOCK.  */
 
 static void
-write_block (current_offset, n_bytes, prev_block, curr_block)
-     long unsigned int current_offset;
-     long unsigned int n_bytes;
-     const char *prev_block;
-     const char *curr_block;
+write_block (long unsigned int current_offset, long unsigned int n_bytes,
+	     const char *prev_block, const char *curr_block)
 {
   static int first = 1;
   static int prev_pair_equal = 0;
@@ -1135,13 +1101,13 @@ write_block (current_offset, n_bytes, prev_block, curr_block)
 }
 
 /* Test whether there have been errors on in_stream, and close it if
-   it is not standard input.  Return non-zero if there has been an error
+   it is not standard input.  Return nonzero if there has been an error
    on in_stream or stdout; return zero otherwise.  This function will
    report more than one error only if both a read and a write error
    have occurred.  */
 
 static int
-check_and_close ()
+check_and_close (void)
 {
   int err;
 
@@ -1177,12 +1143,11 @@ check_and_close ()
    the list.  Then try to read a byte from the newly opened file.
    Repeat if necessary until *FILE_LIST is NULL.  When EOF is reached
    for the last file in FILE_LIST, set *C to EOF and return.  Subsequent
-   calls do likewise.  The return value is non-zero if any errors
+   calls do likewise.  The return value is nonzero if any errors
    occured, zero otherwise.  */
 
 static int
-read_char (c)
-     int *c;
+read_char (int *c)
 {
   int err;
 
@@ -1240,14 +1205,11 @@ read_char (c)
    Repeat if necessary until *FILE_LIST is NULL.  Set *N_BYTES_IN_BUFFER
    to the number of bytes read.  If an error occurs, it will be detected
    through ferror when the stream is about to be closed.  If there is an
-   error, give a message but continue reading as usual and return non-zero.
+   error, give a message but continue reading as usual and return nonzero.
    Otherwise return zero.  */
 
 static int
-read_block (n, block, n_bytes_in_buffer)
-     size_t n;
-     char *block;
-     size_t *n_bytes_in_buffer;
+read_block (size_t n, char *block, size_t *n_bytes_in_buffer)
 {
   int err;
 
@@ -1308,7 +1270,7 @@ read_block (n, block, n_bytes_in_buffer)
    with the format specs.  */
 
 static int
-get_lcm ()
+get_lcm (void)
 {
   unsigned int i;
   int l_c_m = 1;
@@ -1322,8 +1284,7 @@ get_lcm ()
    return the offset it denotes.  Otherwise, return -1.  */
 
 off_t
-parse_old_offset (s)
-     const char *s;
+parse_old_offset (const char *s)
 {
   int radix;
   off_t offset;
@@ -1369,11 +1330,11 @@ parse_old_offset (s)
    multiple of all format spec sizes.  Write the final block.  Finally,
    write on a line by itself the offset of the byte after the last byte
    read.  Accumulate return values from calls to read_block and
-   check_and_close, and if any was non-zero, return non-zero.
+   check_and_close, and if any was nonzero, return nonzero.
    Otherwise, return zero.  */
 
 static int
-dump ()
+dump (void)
 {
   char *block[2];
   off_t current_offset;
@@ -1462,11 +1423,11 @@ dump ()
    A string constant is a run of at least `string_min' ASCII
    graphic (or formatting) characters terminated by a null.
    Based on a function written by Richard Stallman for a
-   pre-POSIX version of od.  Return non-zero if an error
+   pre-POSIX version of od.  Return nonzero if an error
    occurs.  Otherwise, return zero.  */
 
 static int
-dump_strings ()
+dump_strings (void)
 {
   size_t bufsize = MAX (100, string_min);
   char *buf = xmalloc (bufsize);
@@ -1581,9 +1542,7 @@ dump_strings ()
 }
 
 int
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   int c;
   int n_files;
