@@ -31,8 +31,9 @@
 #include "version.h"
 #include "long-options.h"
 #include "error.h"
+#include "xstrtod.h"
 
-#ifdef _POSIX_VERSION
+#ifdef HAVE_LIMITS_H
 #include <limits.h>
 #else
 #ifndef UCHAR_MAX
@@ -45,8 +46,6 @@ char *realloc ();
 void free ();
 #endif
 
-static void usage ();
-
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef min
 #define min(a, b) ((a) < (b) ? (a) : (b))
@@ -61,6 +60,59 @@ static void usage ();
 /* The kind of blanks for '-b' to skip in various options. */
 enum blanktype { bl_start, bl_end, bl_both };
 
+/* Lines are held in core as counted strings. */
+struct line
+{
+  char *text;			/* Text of the line. */
+  int length;			/* Length not including final newline. */
+  char *keybeg;			/* Start of first key. */
+  char *keylim;			/* Limit of first key. */
+};
+
+/* Arrays of lines. */
+struct lines
+{
+  struct line *lines;		/* Dynamically allocated array of lines. */
+  int used;			/* Number of slots used. */
+  int alloc;			/* Number of slots allocated. */
+  int limit;			/* Max number of slots to allocate.  */
+};
+
+/* Input buffers. */
+struct buffer
+{
+  char *buf;			/* Dynamically allocated buffer. */
+  int used;			/* Number of bytes used. */
+  int alloc;			/* Number of bytes allocated. */
+  int left;			/* Number of bytes left after line parsing. */
+};
+
+struct keyfield
+{
+  int sword;			/* Zero-origin 'word' to start at. */
+  int schar;			/* Additional characters to skip. */
+  int skipsblanks;		/* Skip leading white space at start. */
+  int eword;			/* Zero-origin first word after field. */
+  int echar;			/* Additional characters in field. */
+  int skipeblanks;		/* Skip trailing white space at finish. */
+  int *ignore;			/* Boolean array of characters to ignore. */
+  char *translate;		/* Translation applied to characters. */
+  int numeric;			/* Flag for numeric comparison.  Handle
+				   strings of digits with optional decimal
+				   point, but no exponential notation. */
+  int general_numeric;		/* Flag for general, numeric comparison.
+				   Handle numbers in exponential notation. */
+  int month;			/* Flag for comparison by month name. */
+  int reverse;			/* Reverse the sense of comparison. */
+  struct keyfield *next;	/* Next keyfield to try. */
+};
+
+struct month
+{
+  char *name;
+  int val;
+};
+
 /* The name this program was run with. */
 char *program_name;
 
@@ -81,11 +133,7 @@ static char fold_toupper[UCHAR_LIM];
 
 /* Table mapping 3-letter month names to integers.
    Alphabetic order allows binary search. */
-static struct month
-{
-  char *name;
-  int val;
-} const monthtab[] =
+static struct month const monthtab[] =
 {
   {"APR", 4},
   {"AUG", 8},
@@ -141,49 +189,55 @@ static int unique;
 /* Nonzero if any of the input files are the standard input. */
 static int have_read_stdin;
 
-/* Lines are held in core as counted strings. */
-struct line
-{
-  char *text;			/* Text of the line. */
-  int length;			/* Length not including final newline. */
-  char *keybeg;			/* Start of first key. */
-  char *keylim;			/* Limit of first key. */
-};
-
-/* Arrays of lines. */
-struct lines
-{
-  struct line *lines;		/* Dynamically allocated array of lines. */
-  int used;			/* Number of slots used. */
-  int alloc;			/* Number of slots allocated. */
-  int limit;			/* Max number of slots to allocate.  */
-};
-
-/* Input buffers. */
-struct buffer
-{
-  char *buf;			/* Dynamically allocated buffer. */
-  int used;			/* Number of bytes used. */
-  int alloc;			/* Number of bytes allocated. */
-  int left;			/* Number of bytes left after line parsing. */
-};
-
 /* Lists of key field comparisons to be tried. */
-static struct keyfield
+static struct keyfield keyhead;
+
+static void
+usage (int status)
 {
-  int sword;			/* Zero-origin 'word' to start at. */
-  int schar;			/* Additional characters to skip. */
-  int skipsblanks;		/* Skip leading white space at start. */
-  int eword;			/* Zero-origin first word after field. */
-  int echar;			/* Additional characters in field. */
-  int skipeblanks;		/* Skip trailing white space at finish. */
-  int *ignore;			/* Boolean array of characters to ignore. */
-  char *translate;		/* Translation applied to characters. */
-  int numeric;			/* Flag for numeric comparison. */
-  int month;			/* Flag for comparison by month name. */
-  int reverse;			/* Reverse the sense of comparison. */
-  struct keyfield *next;	/* Next keyfield to try. */
-} keyhead;
+  if (status != 0)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
+  else
+    {
+      printf (_("\
+Usage: %s [OPTION]... [FILE]...\n\
+"),
+	      program_name);
+      printf (_("\
+Write sorted concatenation of all FILE(s) to standard output.\n\
+\n\
+  +POS1 [-POS2]    start a key at POS1, end it before POS2\n\
+  -M               compare (unknown) < `JAN' < ... < `DEC', imply -b\n\
+  -T DIRECT        use DIRECT for temporary files, not $TMPDIR or %s\n\
+  -b               ignore leading blanks in sort fields or keys\n\
+  -c               check if given files already sorted, do not sort\n\
+  -d               consider only [a-zA-Z0-9 ] characters in keys\n\
+  -f               fold lower case to upper case characters in keys\n\
+  -g               compare according to general numerical value, imply -b\n\
+  -i               consider only [\\040-\\0176] characters in keys\n\
+  -k POS1[,POS2]   same as +POS1 [-POS2], but all positions counted from 1\n\
+  -m               merge already sorted files, do not sort\n\
+  -n               compare according to string numerical value, imply -b\n\
+  -o FILE          write result on FILE instead of standard output\n\
+  -r               reverse the result of comparisons\n\
+  -s               stabilize sort by disabling last resort comparison\n\
+  -t SEP           use SEParator instead of non- to whitespace transition\n\
+  -u               with -c, check for strict ordering\n\
+  -u               with -m, only output the first of an equal sequence\n\
+      --help       display this help and exit\n\
+      --version    output version information and exit\n\
+\n\
+POS is F[.C][OPTS], where F is the field number and C the character\n\
+position in the field, both counted from zero.  OPTS is made up of one\n\
+or more of Mbdfinr, this effectively disable global -Mbdfinr settings\n\
+for that key.  If no key given, use the entire line as key.  With no\n\
+FILE, or when FILE is -, read standard input.\n\
+")
+	      , DEFAULT_TMPDIR);
+    }
+  exit (status);
+}
 
 /* The list of temporary files. */
 static struct tempnode
@@ -195,7 +249,7 @@ static struct tempnode
 /* Clean up any remaining temporary files. */
 
 static void
-cleanup ()
+cleanup (void)
 {
   struct tempnode *node;
 
@@ -205,9 +259,8 @@ cleanup ()
 
 /* Allocate N bytes of memory dynamically, with error checking.  */
 
-char *
-xmalloc (n)
-     unsigned n;
+static char *
+xmalloc (unsigned int n)
 {
   char *p;
 
@@ -226,10 +279,8 @@ xmalloc (n)
    If P is NULL, run xmalloc.
    If N is 0, run free and return NULL.  */
 
-char *
-xrealloc (p, n)
-     char *p;
-     unsigned n;
+static char *
+xrealloc (char *p, unsigned int n)
 {
   if (p == 0)
     return xmalloc (n);
@@ -249,25 +300,48 @@ xrealloc (p, n)
 }
 
 static FILE *
-xfopen (file, how)
-     char *file, *how;
+xtmpfopen (const char *file)
 {
-  FILE *fp = strcmp (file, "-") ? fopen (file, how) : stdin;
+  FILE *fp;
+  int fd;
 
-  if (fp == 0)
+  fd = open (file, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+  if (fd < 0 || (fp = fdopen (fd, "w")) == NULL)
     {
       error (0, errno, "%s", file);
       cleanup ();
       exit (2);
     }
+
+  return fp;
+}
+
+static FILE *
+xfopen (const char *file, const char *how)
+{
+  FILE *fp;
+
+  if (strcmp (file, "-") == 0)
+    {
+      fp = stdin;
+    }
+  else
+    {
+      if ((fp = fopen (file, how)) == NULL)
+	{
+	  error (0, errno, "%s", file);
+	  cleanup ();
+	  exit (2);
+	}
+    }
+
   if (fp == stdin)
     have_read_stdin = 1;
   return fp;
 }
 
 static void
-xfclose (fp)
-     FILE *fp;
+xfclose (FILE *fp)
 {
   if (fp == stdin)
     {
@@ -296,10 +370,7 @@ xfclose (fp)
 }
 
 static void
-xfwrite (buf, size, nelem, fp)
-     char *buf;
-     int size, nelem;
-     FILE *fp;
+xfwrite (const char *buf, int size, int nelem, FILE *fp)
 {
   if (fwrite (buf, size, nelem, fp) != nelem)
     {
@@ -312,7 +383,7 @@ xfwrite (buf, size, nelem, fp)
 /* Return a name for a temporary file. */
 
 static char *
-tempname ()
+tempname (void)
 {
   static unsigned int seq;
   int len = strlen (temp_file_prefix);
@@ -341,8 +412,7 @@ tempname ()
    remove it if it is found on the list. */
 
 static void
-zaptemp (name)
-     char *name;
+zaptemp (char *name)
 {
   struct tempnode *node, *temp;
 
@@ -362,7 +432,7 @@ zaptemp (name)
 /* Initialize the character class tables. */
 
 static void
-inittables ()
+inittables (void)
 {
   int i;
 
@@ -386,9 +456,7 @@ inittables ()
 /* Initialize BUF, allocating ALLOC bytes initially. */
 
 static void
-initbuf (buf, alloc)
-     struct buffer *buf;
-     int alloc;
+initbuf (struct buffer *buf, int alloc)
 {
   buf->alloc = alloc;
   buf->buf = xmalloc (buf->alloc);
@@ -401,9 +469,7 @@ initbuf (buf, alloc)
    of bytes buffered. */
 
 static int
-fillbuf (buf, fp)
-     struct buffer *buf;
-     FILE *fp;
+fillbuf (struct buffer *buf, FILE *fp)
 {
   int cc;
 
@@ -445,10 +511,7 @@ fillbuf (buf, fp)
    for, ever.  */
 
 static void
-initlines (lines, alloc, limit)
-     struct lines *lines;
-     int alloc;
-     int limit;
+initlines (struct lines *lines, int alloc, int limit)
 {
   lines->alloc = alloc;
   lines->lines = (struct line *) xmalloc (lines->alloc * sizeof (struct line));
@@ -460,9 +523,7 @@ initlines (lines, alloc, limit)
    by KEY in LINE. */
 
 static char *
-begfield (line, key)
-     struct line *line;
-     struct keyfield *key;
+begfield (const struct line *line, const struct keyfield *key)
 {
   register char *ptr = line->text, *lim = ptr + line->length;
   register int sword = key->sword, schar = key->schar;
@@ -488,8 +549,10 @@ begfield (line, key)
     while (ptr < lim && blanks[UCHAR (*ptr)])
       ++ptr;
 
-  while (ptr < lim && schar--)
-    ++ptr;
+  if (ptr + schar <= lim)
+    ptr += schar;
+  else
+    ptr = lim;
 
   return ptr;
 }
@@ -498,19 +561,28 @@ begfield (line, key)
    in LINE specified by KEY. */
 
 static char *
-limfield (line, key)
-     struct line *line;
-     struct keyfield *key;
+limfield (const struct line *line, const struct keyfield *key)
 {
   register char *ptr = line->text, *lim = ptr + line->length;
   register int eword = key->eword, echar = key->echar;
 
+  /* Note: from the POSIX spec:
+     The leading field separator itself is included in
+     a field when -t is not used.  FIXME: move this comment up... */
+
+  /* Move PTR past EWORD fields or to one past the last byte on LINE,
+     whichever comes first.  If there are more than EWORD fields, leave
+     PTR pointing at the beginning of the field having zero-based index,
+     EWORD.  If a delimiter character was specified (via -t), then that
+     `beginning' is the first character following the delimiting TAB.
+     Otherwise, leave PTR pointing at the first `blank' character after
+     the preceding field.  */
   if (tab)
     while (ptr < lim && eword--)
       {
 	while (ptr < lim && *ptr != tab)
 	  ++ptr;
-	if (ptr < lim && (eword || key->skipeblanks))
+	if (ptr < lim && (eword || echar > 0))
 	  ++ptr;
       }
   else
@@ -522,23 +594,54 @@ limfield (line, key)
 	  ++ptr;
       }
 
-  if (key->skipeblanks)
+  /* Make LIM point to the end of (one byte past) the current field.  */
+  if (tab)
+    {
+      char *newlim;
+      newlim = memchr (ptr, tab, lim - ptr);
+      if (newlim)
+	lim = newlim;
+    }
+  else
+    {
+      char *newlim;
+      newlim = ptr;
+      while (newlim < lim && blanks[UCHAR (*newlim)])
+	++newlim;
+      while (newlim < lim && !blanks[UCHAR (*newlim)])
+	++newlim;
+      lim = newlim;
+    }
+
+  /* If we're skipping leading blanks, don't start counting characters
+     until after skipping past any leading blanks.  */
+  if (key->skipsblanks)
     while (ptr < lim && blanks[UCHAR (*ptr)])
       ++ptr;
 
-  while (ptr < lim && echar--)
-    ++ptr;
+  /* Advance PTR by ECHAR (if possible), but no further than LIM.  */
+  if (ptr + echar <= lim)
+    ptr += echar;
+  else
+    ptr = lim;
 
   return ptr;
 }
 
+/* FIXME */
+
+void
+trim_trailing_blanks (const char *a_start, char **a_end)
+{
+  while (*a_end > a_start && blanks[UCHAR (*(*a_end - 1))])
+    --(*a_end);
+}
+
 /* Find the lines in BUF, storing pointers and lengths in LINES.
-   Also replace newlines with NULs. */
+   Also replace newlines in BUF with NULs. */
 
 static void
-findlines (buf, lines)
-     struct buffer *buf;
-     struct lines *lines;
+findlines (struct buffer *buf, struct lines *lines)
 {
   register char *beg = buf->buf, *lim = buf->buf + buf->used, *ptr;
   struct keyfield *key = keyhead.next;
@@ -583,6 +686,11 @@ findlines (buf, lines)
 		  ++beg;
 	      lines->lines[lines->used].keybeg = beg;
 	    }
+	  if (key->skipeblanks)
+	    {
+	      trim_trailing_blanks (lines->lines[lines->used].keybeg,
+				    &lines->lines[lines->used].keylim);
+	    }
 	}
       else
 	{
@@ -602,8 +710,7 @@ findlines (buf, lines)
    of the fraction.  Strings not of this form are considered to be zero. */
 
 static int
-fraccompare (a, b)
-     register char *a, *b;
+fraccompare (register const char *a, register const char *b)
 {
   register tmpa = UCHAR (*a), tmpb = UCHAR (*b);
 
@@ -658,8 +765,7 @@ fraccompare (a, b)
    hideously fast. */
 
 static int
-numcompare (a, b)
-     register char *a, *b;
+numcompare (register const char *a, register const char *b)
 {
   register int tmpa, tmpb, loga, logb, tmp;
 
@@ -673,19 +779,30 @@ numcompare (a, b)
 
   if (tmpa == '-')
     {
-      tmpa = UCHAR (*++a);
+      do
+	tmpa = UCHAR (*++a);
+      while (tmpa == '0');
       if (tmpb != '-')
 	{
-	  if (digits[tmpa] && digits[tmpb])
+	  if (tmpa == '.')
+	    do
+	      tmpa = UCHAR (*++a);
+	    while (tmpa == '0');
+	  if (digits[tmpa])
+	    return -1;
+	  while (tmpb == '0')
+	    tmpb = UCHAR (*++b);
+	  if (tmpb == '.')
+	    do
+	      tmpb = *++b;
+	    while (tmpb == '0');
+	  if (digits[tmpb])
 	    return -1;
 	  return 0;
 	}
-      tmpb = UCHAR (*++b);
-
-      while (tmpa == '0')
-	tmpa = UCHAR (*++a);
-      while (tmpb == '0')
+      do
 	tmpb = UCHAR (*++b);
+      while (tmpb == '0');
 
       while (tmpa == tmpb && digits[tmpa])
 	tmpa = UCHAR (*++a), tmpb = UCHAR (*++b);
@@ -715,7 +832,22 @@ numcompare (a, b)
     }
   else if (tmpb == '-')
     {
-      if (digits[UCHAR (tmpa)] && digits[UCHAR (*++b)])
+      do
+	tmpb = UCHAR (*++b);
+      while (tmpb == '0');
+      if (tmpb == '.')
+	do
+	  tmpb = *++b;
+	while (tmpb == '0');
+      if (digits[tmpb])
+	return 1;
+      while (tmpa == '0')
+	tmpa = UCHAR (*++a);
+      if (tmpa == '.')
+	do
+	  tmpa = UCHAR (*++a);
+	while (tmpa == '0');
+      if (digits[tmpa])
 	return 1;
       return 0;
     }
@@ -754,13 +886,29 @@ numcompare (a, b)
     }
 }
 
+static int
+general_numcompare (const char *sa, const char *sb)
+{
+  double a, b;
+  /* FIXME: add option to warn about failed conversions.  */
+  /* FIXME: maybe add option to try expensive FP conversion
+     only if A and B can't be compared more cheaply/accurately.  */
+  if (xstrtod (sa, NULL, &a))
+    {
+      a = 0;
+    }
+  if (xstrtod (sb, NULL, &b))
+    {
+      b = 0;
+    }
+  return a == b ? 0 : a < b ? -1 : 1;
+}
+
 /* Return an integer <= 12 associated with month name S with length LEN,
    0 if the name in S is not recognized. */
 
 static int
-getmonth (s, len)
-     char *s;
-     int len;
+getmonth (const char *s, int len)
 {
   char month[4];
   register int i, lo = 0, hi = 12;
@@ -789,8 +937,7 @@ getmonth (s, len)
    are no more keys or a difference is found. */
 
 static int
-keycompare (a, b)
-     struct line *a, *b;
+keycompare (const struct line *a, const struct line *b)
 {
   register char *texta, *textb, *lima, *limb, *translate;
   register int *ignore;
@@ -839,6 +986,16 @@ keycompare (a, b)
       if (lenb < 0)
 	lenb = 0;
 
+      if (key->skipeblanks)
+        {
+	  char *a_end = texta + lena;
+	  char *b_end = textb + lenb;
+	  trim_trailing_blanks (texta, &a_end);
+	  trim_trailing_blanks (textb, &b_end);
+	  lena = a_end - texta;
+	  lenb = b_end - textb;
+	}
+
       /* Actually compare the fields. */
       if (key->numeric)
 	{
@@ -853,6 +1010,23 @@ keycompare (a, b)
 	  else
 	    diff = numcompare (texta, textb);
 
+	  if (diff)
+	    return key->reverse ? -diff : diff;
+	  continue;
+	}
+      else if (key->general_numeric)
+	{
+	  if (*lima || *limb)
+	    {
+	      char savea = *lima, saveb = *limb;
+
+	      *lima = *limb = '\0';
+	      diff = general_numcompare (texta, textb);
+	      *lima = savea, *limb = saveb;
+	    }
+	  else
+	    diff = general_numcompare (texta, textb);
+
 	  if (diff)
 	    return key->reverse ? -diff : diff;
 	  continue;
@@ -865,42 +1039,64 @@ keycompare (a, b)
 	  continue;
 	}
       else if (ignore && translate)
-	while (texta < lima && textb < limb)
-	  {
-	    while (texta < lima && ignore[UCHAR (*texta)])
-	      ++texta;
-	    while (textb < limb && ignore[UCHAR (*textb)])
-	      ++textb;
-	    if (texta < lima && textb < limb &&
-		translate[UCHAR (*texta++)] != translate[UCHAR (*textb++)])
-	      {
-		diff = translate[UCHAR (*--texta)] - translate[UCHAR (*--textb)];
-		break;
-	      }
-	    else if (texta == lima && textb < limb) diff = -1;
-	    else if (texta < lima && textb == limb) diff = 1;
-	  }
+
+#define CMP_WITH_IGNORE(A, B)						\
+  do									\
+    {									\
+	  while (texta < lima && textb < limb)				\
+	    {								\
+	      while (texta < lima && ignore[UCHAR (*texta)])		\
+		++texta;						\
+	      while (textb < limb && ignore[UCHAR (*textb)])		\
+		++textb;						\
+	      if (texta < lima && textb < limb)				\
+		{							\
+		  if ((A) != (B))					\
+		    {							\
+		      diff = (A) - (B);					\
+		      break;						\
+		    }							\
+		  ++texta;						\
+		  ++textb;						\
+		}							\
+									\
+	      if (texta == lima && textb < limb && !ignore[UCHAR (*textb)]) \
+		diff = -1;						\
+	      else if (texta < lima && textb == limb			\
+		       && !ignore[UCHAR (*texta)])			\
+		diff = 1;						\
+	    }								\
+									\
+	  if (diff == 0)						\
+	    {								\
+	      while (texta < lima && ignore[UCHAR (*texta)])		\
+		++texta;						\
+	      while (textb < limb && ignore[UCHAR (*textb)])		\
+		++textb;						\
+									\
+	      if (texta == lima && textb < limb)			\
+		diff = -1;						\
+	      else if (texta < lima && textb == limb)			\
+		diff = 1;						\
+	    }								\
+	  /* Relative lengths are meaningless if characters were ignored.  \
+	     Handling this case here avoids what might be an invalid length  \
+	     comparison below.  */					\
+	  if (diff == 0 && texta == lima && textb == limb)		\
+	    return 0;							\
+    }									\
+  while (0)
+
+	CMP_WITH_IGNORE (translate[UCHAR (*texta)], translate[UCHAR (*textb)]);
       else if (ignore)
-	while (texta < lima && textb < limb)
-	  {
-	    while (texta < lima && ignore[UCHAR (*texta)])
-	      ++texta;
-	    while (textb < limb && ignore[UCHAR (*textb)])
-	      ++textb;
-	    if (texta < lima && textb < limb && *texta++ != *textb++)
-	      {
-		diff = *--texta - *--textb;
-		break;
-	      }
-	    else if (texta == lima && textb < limb) diff = -1;
-	    else if (texta < lima && textb == limb) diff = 1;
-	  }
+	CMP_WITH_IGNORE (*texta, *textb);
       else if (translate)
 	while (texta < lima && textb < limb)
 	  {
 	    if (translate[UCHAR (*texta++)] != translate[UCHAR (*textb++)])
 	      {
-		diff = translate[UCHAR (*--texta)] - translate[UCHAR (*--textb)];
+		diff = (translate[UCHAR (*--texta)]
+			- translate[UCHAR (*--textb)]);
 		break;
 	      }
 	  }
@@ -920,8 +1116,7 @@ keycompare (a, b)
    depending on whether A compares less than, equal to, or greater than B. */
 
 static int
-compare (a, b)
-     register struct line *a, *b;
+compare (register const struct line *a, register const struct line *b)
 {
   int diff, tmpa, tmpb, mini;
 
@@ -964,8 +1159,7 @@ compare (a, b)
    FIXME: return number of first out-of-order line if not sorted.  */
 
 static int
-checkfp (fp)
-     FILE *fp;
+checkfp (FILE *fp)
 {
   struct buffer buf;		/* Input buffer. */
   struct lines lines;		/* Lines scanned from the buffer. */
@@ -1042,9 +1236,7 @@ finish:
    Close FPS before returning. */
 
 static void
-mergefps (fps, nfps, ofp)
-     FILE *fps[], *ofp;
-     register int nfps;
+mergefps (FILE **fps, register int nfps, FILE *ofp)
 {
   struct buffer buffer[NMERGE];	/* Input buffers for each file. */
   struct lines lines[NMERGE];	/* Line tables for each buffer. */
@@ -1206,9 +1398,7 @@ mergefps (fps, nfps, ofp)
 /* Sort the array LINES with NLINES members, using TEMP for temporary space. */
 
 static void
-sortlines (lines, nlines, temp)
-     struct line *lines, *temp;
-     int nlines;
+sortlines (struct line *lines, int nlines, struct line *temp)
 {
   register struct line *lo, *hi, *t;
   register int nlo, nhi;
@@ -1249,9 +1439,7 @@ sortlines (lines, nlines, temp)
    Return a count of disordered files. */
 
 static int
-check (files, nfiles)
-     char *files[];
-     int nfiles;
+check (char **files, int nfiles)
 {
   int i, disorders = 0;
   FILE *fp;
@@ -1271,10 +1459,7 @@ check (files, nfiles)
 /* Merge NFILES FILES onto OFP. */
 
 static void
-merge (files, nfiles, ofp)
-     char *files[];
-     int nfiles;
-     FILE *ofp;
+merge (char **files, int nfiles, FILE *ofp)
 {
   int i, j, t;
   char *temp;
@@ -1287,7 +1472,7 @@ merge (files, nfiles, ofp)
 	{
 	  for (j = 0; j < NMERGE; ++j)
 	    fps[j] = xfopen (files[i * NMERGE + j], "r");
-	  tfp = xfopen (temp = tempname (), "w");
+	  tfp = xtmpfopen (temp = tempname ());
 	  mergefps (fps, NMERGE, tfp);
 	  xfclose (tfp);
 	  for (j = 0; j < NMERGE; ++j)
@@ -1296,7 +1481,7 @@ merge (files, nfiles, ofp)
 	}
       for (j = 0; j < nfiles % NMERGE; ++j)
 	fps[j] = xfopen (files[i * NMERGE + j], "r");
-      tfp = xfopen (temp = tempname (), "w");
+      tfp = xtmpfopen (temp = tempname ());
       mergefps (fps, nfiles % NMERGE, tfp);
       xfclose (tfp);
       for (j = 0; j < nfiles % NMERGE; ++j)
@@ -1315,10 +1500,7 @@ merge (files, nfiles, ofp)
 /* Sort NFILES FILES onto OFP. */
 
 static void
-sort (files, nfiles, ofp)
-     char **files;
-     int nfiles;
-     FILE *ofp;
+sort (char **files, int nfiles, FILE *ofp)
 {
   struct buffer buf;
   struct lines lines;
@@ -1354,7 +1536,7 @@ sort (files, nfiles, ofp)
 	  else
 	    {
 	      ++n_temp_files;
-	      tfp = xfopen (tempname (), "w");
+	      tfp = xtmpfopen (tempname ());
 	    }
 	  for (i = 0; i < lines.used; ++i)
 	    if (!unique || i == 0
@@ -1387,8 +1569,7 @@ sort (files, nfiles, ofp)
 /* Insert key KEY at the end of the list (`keyhead'). */
 
 static void
-insertkey (key)
-     struct keyfield *key;
+insertkey (struct keyfield *key)
 {
   struct keyfield *k = &keyhead;
 
@@ -1399,8 +1580,7 @@ insertkey (key)
 }
 
 static void
-badfieldspec (s)
-     char *s;
+badfieldspec (const char *s)
 {
   error (2, 0, _("invalid field specification `%s'"), s);
 }
@@ -1408,19 +1588,18 @@ badfieldspec (s)
 /* Handle interrupts and hangups. */
 
 static void
-sighandler (sig)
-     int sig;
+sighandler (int sig)
 {
-#ifdef _POSIX_VERSION
+#ifdef SA_INTERRUPT
   struct sigaction sigact;
 
   sigact.sa_handler = SIG_DFL;
   sigemptyset (&sigact.sa_mask);
   sigact.sa_flags = 0;
   sigaction (sig, &sigact, NULL);
-#else				/* !_POSIX_VERSION */
+#else				/* !SA_INTERRUPT */
   signal (sig, SIG_DFL);
-#endif				/* _POSIX_VERSION */
+#endif				/* SA_INTERRUPT */
   cleanup ();
   kill (getpid (), sig);
 }
@@ -1431,10 +1610,8 @@ sighandler (sig)
    BLANKTYPE is the kind of blanks that 'b' should skip. */
 
 static char *
-set_ordering (s, key, blanktype)
-     register char *s;
-     struct keyfield *key;
-     enum blanktype blanktype;
+set_ordering (register const char *s, struct keyfield *key,
+	      enum blanktype blanktype)
 {
   while (*s)
     {
@@ -1452,11 +1629,9 @@ set_ordering (s, key, blanktype)
 	case 'f':
 	  key->translate = fold_toupper;
 	  break;
-#if 0
 	case 'g':
-	  /* Reserved for comparing floating-point numbers. */
+	  key->general_numeric = 1;
 	  break;
-#endif
 	case 'i':
 	  key->ignore = nonprinting;
 	  break;
@@ -1474,17 +1649,15 @@ set_ordering (s, key, blanktype)
 	  key->reverse = 1;
 	  break;
 	default:
-	  return s;
+	  return (char *) s;
 	}
       ++s;
     }
-  return s;
+  return (char *) s;
 }
 
 void
-main (argc, argv)
-     int argc;
-     char *argv[];
+main (int argc, char **argv)
 {
   struct keyfield *key = NULL, gkey;
   char *s;
@@ -1492,9 +1665,9 @@ main (argc, argv)
   int checkonly = 0, mergeonly = 0, nfiles = 0;
   char *minus = "-", *outfile = minus, **files, *tmp;
   FILE *ofp;
-#ifdef _POSIX_VERSION
+#ifdef SA_INTERRUPT
   struct sigaction oldact, newact;
-#endif				/* _POSIX_VERSION */
+#endif				/* SA_INTERRUPT */
 
   program_name = argv[0];
 
@@ -1507,7 +1680,7 @@ main (argc, argv)
   if (temp_file_prefix == NULL)
     temp_file_prefix = DEFAULT_TMPDIR;
 
-#ifdef _POSIX_VERSION
+#ifdef SA_INTERRUPT
   newact.sa_handler = sighandler;
   sigemptyset (&newact.sa_mask);
   newact.sa_flags = 0;
@@ -1524,7 +1697,7 @@ main (argc, argv)
   sigaction (SIGTERM, NULL, &oldact);
   if (oldact.sa_handler != SIG_IGN)
     sigaction (SIGTERM, &newact, NULL);
-#else				/* !_POSIX_VERSION */
+#else				/* !SA_INTERRUPT */
   if (signal (SIGINT, SIG_IGN) != SIG_IGN)
     signal (SIGINT, sighandler);
   if (signal (SIGHUP, SIG_IGN) != SIG_IGN)
@@ -1533,12 +1706,12 @@ main (argc, argv)
     signal (SIGPIPE, sighandler);
   if (signal (SIGTERM, SIG_IGN) != SIG_IGN)
     signal (SIGTERM, sighandler);
-#endif				/* !_POSIX_VERSION */
+#endif				/* !SA_INTERRUPT */
 
   gkey.sword = gkey.eword = -1;
   gkey.ignore = NULL;
   gkey.translate = NULL;
-  gkey.numeric = gkey.month = gkey.reverse = 0;
+  gkey.numeric =  gkey.general_numeric = gkey.month = gkey.reverse = 0;
   gkey.skipsblanks = gkey.skipeblanks = 0;
 
   files = (char **) xmalloc (sizeof (char *) * argc);
@@ -1554,7 +1727,7 @@ main (argc, argv)
 	  key->ignore = NULL;
 	  key->translate = NULL;
 	  key->skipsblanks = key->skipeblanks = 0;
-	  key->numeric = key->month = key->reverse = 0;
+	  key->numeric = key->general_numeric = key->month = key->reverse = 0;
 	  s = argv[i] + 1;
 	  if (! (digits[UCHAR (*s)] || (*s == '.' && digits[UCHAR (s[1])])))
 	    badfieldspec (argv[i]);
@@ -1631,15 +1804,34 @@ main (argc, argv)
 		      badfieldspec (argv[i]);
 		    for (t = 0; digits[UCHAR (*s)]; ++s)
 		      t = 10 * t + *s - '0';
-		    if (t)
-		      t--;
+		    if (t == 0)
+		      {
+			/* Provoke with `sort -k0' */
+			error (0, 0, _("the starting field number argument \
+to the `-k' option must be positive"));
+			badfieldspec (argv[i]);
+		      }
+		    --t;
 		    t2 = 0;
 		    if (*s == '.')
 		      {
+			if (!digits[UCHAR (s[1])])
+			  {
+			    /* Provoke with `sort -k1.' */
+			    error (0, 0, _("starting field spec has `.' but \
+lacks following character offset"));
+			    badfieldspec (argv[i]);
+			  }
 			for (++s; digits[UCHAR (*s)]; ++s)
 			  t2 = 10 * t2 + *s - '0';
-			if (t2)
-			  t2--;
+			if (t2 == 0)
+			  {
+			    /* Provoke with `sort -k1.0' */
+			    error (0, 0, _("starting field character offset \
+argument to the `-k' option\nmust be positive"));
+			    badfieldspec (argv[i]);
+			  }
+			--t2;
 		      }
 		    if (t2 || t)
 		      {
@@ -1649,24 +1841,47 @@ main (argc, argv)
 		    else
 		      key->sword = -1;
 		    s = set_ordering (s, key, bl_start);
-		    if (*s && *s != ',')
-		      badfieldspec (argv[i]);
-		    else if (*s++)
+		    if (*s == 0)
 		      {
+			key->eword = -1;
+			key->echar = 0;
+		      }
+		    else if (*s != ',')
+		      badfieldspec (argv[i]);
+		    else if (*s == ',')
+		      {
+			/* Skip over comma.  */
+			++s;
+			if (*s == 0)
+			  {
+			    /* Provoke with `sort -k1,' */
+			    error (0, 0, _("field specification has `,' but \
+lacks following field spec"));
+			    badfieldspec (argv[i]);
+			  }
 			/* Get POS2. */
 			for (t = 0; digits[UCHAR (*s)]; ++s)
 			  t = t * 10 + *s - '0';
-			if (t)
-			      t--;
+			if (t == 0)
+			  {
+			    /* Provoke with `sort -k1,0' */
+			    error (0, 0, _("ending field number argument \
+to the `-k' option must be positive"));
+			    badfieldspec (argv[i]);
+			  }
+			--t;
 			t2 = 0;
-			/* FIXME: It's an error to specify `.'
-			   but no char-spec. */
 			if (*s == '.')
 			  {
+			    if (!digits[UCHAR (s[1])])
+			      {
+				/* Provoke with `sort -k1,1.' */
+				error (0, 0, _("ending field spec has `.' \
+but lacks following character offset"));
+				badfieldspec (argv[i]);
+			      }
 			    for (++s; digits[UCHAR (*s)]; ++s)
 			      t2 = t2 * 10 + *s - '0';
-			    if (t2)
-			      t2--;
 			  }
 			else
 			  {
@@ -1751,7 +1966,8 @@ main (argc, argv)
   /* Inheritance of global options to individual keys. */
   for (key = keyhead.next; key; key = key->next)
     if (!key->ignore && !key->translate && !key->skipsblanks && !key->reverse
-	&& !key->skipeblanks && !key->month && !key->numeric)
+	&& !key->skipeblanks && !key->month && !key->numeric
+        && !key->general_numeric)
       {
 	key->ignore = gkey.ignore;
 	key->translate = gkey.translate;
@@ -1759,11 +1975,13 @@ main (argc, argv)
 	key->skipeblanks = gkey.skipeblanks;
 	key->month = gkey.month;
 	key->numeric = gkey.numeric;
+	key->general_numeric = gkey.general_numeric;
 	key->reverse = gkey.reverse;
       }
 
   if (!keyhead.next && (gkey.ignore || gkey.translate || gkey.skipsblanks
-			|| gkey.skipeblanks || gkey.month || gkey.numeric))
+			|| gkey.skipeblanks || gkey.month || gkey.numeric
+                        || gkey.general_numeric))
     insertkey (&gkey);
   reverse = gkey.reverse;
 
@@ -1818,7 +2036,7 @@ main (argc, argv)
 
 	      fp = xfopen (files[i], "r");
 	      tmp = tempname ();
-	      ofp = xfopen (tmp, "w");
+	      ofp = xtmpfopen (tmp);
 	      while ((cc = fread (buf, 1, sizeof buf, fp)) > 0)
 		xfwrite (buf, 1, cc, ofp);
 	      if (ferror (fp))
@@ -1858,50 +2076,3 @@ main (argc, argv)
 
   exit (0);
 }
-
-static void
-usage (status)
-     int status;
-{
-  if (status != 0)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
-  else
-    {
-      printf (_("\
-Usage: %s [OPTION]... [FILE]...\n\
-"),
-	      program_name);
-      printf (_("\
-Write sorted concatenation of all FILE(s) to standard output.\n\
-\n\
-  +POS1 [-POS2]    start a key at POS1, end it before POS2\n\
-  -M               compare (unknown) < `JAN' < ... < `DEC', imply -b\n\
-  -T DIRECT        use DIRECT for temporary files, not $TMPDIR or %s\n\
-  -b               ignore leading blanks in sort fields or keys\n\
-  -c               check if given files already sorted, do not sort\n\
-  -d               consider only [a-zA-Z0-9 ] characters in keys\n\
-  -f               fold lower case to upper case characters in keys\n\
-  -i               consider only [\\040-\\0176] characters in keys\n\
-  -k POS1[,POS2]   same as +POS1 [-POS2], but all positions counted from 1\n\
-  -m               merge already sorted files, do not sort\n\
-  -n               compare according to string numerical value, imply -b\n\
-  -o FILE          write result on FILE instead of standard output\n\
-  -r               reverse the result of comparisons\n\
-  -s               stabilize sort by disabling last resort comparison\n\
-  -t SEP           use SEParator instead of non- to whitespace transition\n\
-  -u               with -c, check for strict ordering\n\
-  -u               with -m, only output the first of an equal sequence\n\
-      --help       display this help and exit\n\
-      --version    output version information and exit\n\
-\n\
-POS is F[.C][OPTS], where F is the field number and C the character\n\
-position in the field, both counted from zero.  OPTS is made up of one\n\
-or more of Mbdfinr, this effectively disable global -Mbdfinr settings\n\
-for that key.  If no key given, use the entire line as key.  With no\n\
-FILE, or when FILE is -, read standard input.\n\
-")
-	      , DEFAULT_TMPDIR);
-    }
-  exit (status);
-}
