@@ -28,9 +28,6 @@
 #include "version.h"
 #include "error.h"
 
-static int bsd_sum_file ();
-static int sysv_sum_file ();
-
 int safe_read ();
 
 /* The name this program was run with. */
@@ -42,10 +39,10 @@ static int have_read_stdin;
 /* Right-rotate 32-bit integer variable C. */
 #define ROTATE_RIGHT(c) if ((c) & 01) (c) = ((c) >>1) + 0x8000; else (c) >>= 1;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output then exit.  */
+/* If nonzero, print the version on standard output then exit.  */
 static int show_version;
 
 static struct option const longopts[] =
@@ -57,8 +54,7 @@ static struct option const longopts[] =
 };
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
@@ -83,10 +79,127 @@ With no FILE, or when FILE is -, read standard input.\n\
   exit (status);
 }
 
+/* Calculate and print the rotated checksum and the size in 1K blocks
+   of file FILE, or of the standard input if FILE is "-".
+   If PRINT_NAME is >1, print FILE next to the checksum and size.
+   The checksum varies depending on sizeof(int).
+   Return 0 if successful, -1 if an error occurs. */
+
+static int
+bsd_sum_file (const char *file, int print_name)
+{
+  register FILE *fp;
+  register unsigned long checksum = 0; /* The checksum mod 2^16. */
+  register long total_bytes = 0; /* The number of bytes. */
+  register int ch;		/* Each character read. */
+
+  if (!strcmp (file, "-"))
+    {
+      fp = stdin;
+      have_read_stdin = 1;
+    }
+  else
+    {
+      fp = fopen (file, "r");
+      if (fp == NULL)
+	{
+	  error (0, errno, "%s", file);
+	  return -1;
+	}
+    }
+
+  while ((ch = getc (fp)) != EOF)
+    {
+      total_bytes++;
+      ROTATE_RIGHT (checksum);
+      checksum += ch;
+      checksum &= 0xffff;	/* Keep it within bounds. */
+    }
+
+  if (ferror (fp))
+    {
+      error (0, errno, "%s", file);
+      if (strcmp (file, "-"))
+	fclose (fp);
+      return -1;
+    }
+
+  if (strcmp (file, "-") && fclose (fp) == EOF)
+    {
+      error (0, errno, "%s", file);
+      return -1;
+    }
+
+  printf ("%05lu %5ld", checksum, (total_bytes + 1024 - 1) / 1024);
+  if (print_name > 1)
+    printf (" %s", file);
+  putchar ('\n');
+
+  return 0;
+}
+
+/* Calculate and print the checksum and the size in 512-byte blocks
+   of file FILE, or of the standard input if FILE is "-".
+   If PRINT_NAME is >0, print FILE next to the checksum and size.
+   Return 0 if successful, -1 if an error occurs. */
+
+static int
+sysv_sum_file (const char *file, int print_name)
+{
+  int fd;
+  unsigned char buf[8192];
+  register int bytes_read;
+  register unsigned long checksum = 0;
+  long total_bytes = 0;
+
+  if (!strcmp (file, "-"))
+    {
+      fd = 0;
+      have_read_stdin = 1;
+    }
+  else
+    {
+      fd = open (file, O_RDONLY);
+      if (fd == -1)
+	{
+	  error (0, errno, "%s", file);
+	  return -1;
+	}
+    }
+
+  while ((bytes_read = safe_read (fd, buf, sizeof buf)) > 0)
+    {
+      register int i;
+
+      for (i = 0; i < bytes_read; i++)
+	checksum += buf[i];
+      total_bytes += bytes_read;
+    }
+
+  if (bytes_read < 0)
+    {
+      error (0, errno, "%s", file);
+      if (strcmp (file, "-"))
+	close (fd);
+      return -1;
+    }
+
+  if (strcmp (file, "-") && close (fd) == -1)
+    {
+      error (0, errno, "%s", file);
+      return -1;
+    }
+
+  printf ("%lu %ld", checksum % 0xffff, (total_bytes + 512 - 1) / 512);
+  if (print_name)
+    printf (" %s", file);
+  putchar ('\n');
+
+  return 0;
+}
+
 void
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   int errors = 0;
   int optc;
@@ -141,127 +254,3 @@ main (argc, argv)
   exit (errors);
 }
 
-/* Calculate and print the rotated checksum and the size in 1K blocks
-   of file FILE, or of the standard input if FILE is "-".
-   If PRINT_NAME is >1, print FILE next to the checksum and size.
-   The checksum varies depending on sizeof(int).
-   Return 0 if successful, -1 if an error occurs. */
-
-static int
-bsd_sum_file (file, print_name)
-     char *file;
-     int print_name;
-{
-  register FILE *fp;
-  register unsigned long checksum = 0; /* The checksum mod 2^16. */
-  register long total_bytes = 0; /* The number of bytes. */
-  register int ch;		/* Each character read. */
-
-  if (!strcmp (file, "-"))
-    {
-      fp = stdin;
-      have_read_stdin = 1;
-    }
-  else
-    {
-      fp = fopen (file, "r");
-      if (fp == NULL)
-	{
-	  error (0, errno, "%s", file);
-	  return -1;
-	}
-    }
-
-  /* This algorithm seems to depend on sign extension in `ch' in order to
-     give the right results.  Ick.  */
-  while ((ch = getc (fp)) != EOF)
-    {
-      total_bytes++;
-      ROTATE_RIGHT (checksum);
-      checksum += ch;
-      checksum &= 0xffff;	/* Keep it within bounds. */
-    }
-
-  if (ferror (fp))
-    {
-      error (0, errno, "%s", file);
-      if (strcmp (file, "-"))
-	fclose (fp);
-      return -1;
-    }
-
-  if (strcmp (file, "-") && fclose (fp) == EOF)
-    {
-      error (0, errno, "%s", file);
-      return -1;
-    }
-
-  printf ("%05lu %5ld", checksum, (total_bytes + 1024 - 1) / 1024);
-  if (print_name > 1)
-    printf (" %s", file);
-  putchar ('\n');
-
-  return 0;
-}
-
-/* Calculate and print the checksum and the size in 512-byte blocks
-   of file FILE, or of the standard input if FILE is "-".
-   If PRINT_NAME is >0, print FILE next to the checksum and size.
-   Return 0 if successful, -1 if an error occurs. */
-
-static int
-sysv_sum_file (file, print_name)
-     char *file;
-     int print_name;
-{
-  int fd;
-  unsigned char buf[8192];
-  register int bytes_read;
-  register unsigned long checksum = 0;
-  long total_bytes = 0;
-
-  if (!strcmp (file, "-"))
-    {
-      fd = 0;
-      have_read_stdin = 1;
-    }
-  else
-    {
-      fd = open (file, O_RDONLY);
-      if (fd == -1)
-	{
-	  error (0, errno, "%s", file);
-	  return -1;
-	}
-    }
-
-  while ((bytes_read = safe_read (fd, buf, sizeof buf)) > 0)
-    {
-      register int i;
-
-      for (i = 0; i < bytes_read; i++)
-	checksum += buf[i];
-      total_bytes += bytes_read;
-    }
-
-  if (bytes_read < 0)
-    {
-      error (0, errno, "%s", file);
-      if (strcmp (file, "-"))
-	close (fd);
-      return -1;
-    }
-
-  if (strcmp (file, "-") && close (fd) == -1)
-    {
-      error (0, errno, "%s", file);
-      return -1;
-    }
-
-  printf ("%lu %ld", checksum % 0xffff, (total_bytes + 512 - 1) / 512);
-  if (print_name)
-    printf (" %s", file);
-  putchar ('\n');
-
-  return 0;
-}
