@@ -195,3 +195,11 @@ char *getenv ();
 /* Disable string localization for the time being.  */
 #undef _
 #define _(String) String
+
+#ifndef __P
+# if PROTOTYPES
+#  define __P(Args) Args
+# else
+#  define __P(Args) ()
+# endif
+#endif
