@@ -12,8 +12,8 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Jim Meyering, meyering@cs.utexas.edu.  */
 
@@ -29,6 +29,10 @@
 #include <sys/types.h>
 #include <getopt.h>
 
+#if HAVE_LIMITS_H
+# include <limits.h>
+#endif
+
 #include "system.h"
 #include "version.h"
 #include "error.h"
@@ -41,6 +45,14 @@
 #define LONG_MAX ((long int) (ULONG_MAX >> 1))
 #endif
 
+#ifndef UINT_MAX
+# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
+#endif
+
+#ifndef INT_MAX
+# define INT_MAX ((int) (UINT_MAX >> 1))
+#endif
+
 #ifndef UCHAR_MAX
 #define UCHAR_MAX 0xFF
 #endif
@@ -60,9 +72,10 @@ typedef int (*PFI) ();
 
 /* The value for Spec_list->state that indicates to
    get_next that it should initialize the tail pointer.
-   Its value doesn't matter as long as it can't be
-   confused with a valid character code.  */
-#define BEGIN_STATE (2 * N_CHARS)
+   Its value should be as large as possible to avoid conflict
+   a valid value for the state field -- and that may be as
+   large as any valid repeat_count.  */
+#define BEGIN_STATE (INT_MAX - 1)
 
 /* The value for Spec_list->state that indicates to
    get_next that the element pointed to by Spec_list->tail is
@@ -183,7 +196,7 @@ struct Spec_list
     /* The number of [c*] and [c*0] constructs that appear in this spec.  */
     int n_indefinite_repeats;
 
-    /* If n_indefinite_repeats is non-zero, this points to the List_element
+    /* If n_indefinite_repeats is nonzero, this points to the List_element
        corresponding to the last [c*] or [c*0] construct encountered in
        this spec.  Otherwise it is undefined.  */
     struct List_element *indefinite_repeat_element;
@@ -201,6 +214,22 @@ struct Spec_list
     int has_restricted_char_class;
   };
 
+/* A representation for escaped string1 or string2.  As a string is parsed,
+   any backslash-escaped characters (other than octal or \a, \b, \f, \n,
+   etc.) are marked as such in this structure by setting the corresponding
+   entry in the ESCAPED vector.  */
+struct E_string
+{
+  unsigned char *s;
+  int *escaped;
+  size_t len;
+};
+
+/* Return nonzero if the Ith character of escaped string ES matches C
+   and is not escaped itself.  */
+#define ES_MATCH(ES, I, C) ((ES)->s[(I)] == (C) && !(ES)->escaped[(I)])
+
+
 char *xmalloc ();
 char *stpcpy ();
 int safe_read ();
@@ -208,29 +237,29 @@ int safe_read ();
 /* The name by which this program was run.  */
 char *program_name;
 
-/* When non-zero, each sequence in the input of a repeated character
+/* When nonzero, each sequence in the input of a repeated character
    (call it c) is replaced (in the output) by a single occurrence of c
    for every c in the squeeze set.  */
 static int squeeze_repeats = 0;
 
-/* When non-zero, removes characters in the delete set from input.  */
+/* When nonzero, removes characters in the delete set from input.  */
 static int delete = 0;
 
 /* Use the complement of set1 in place of set1.  */
 static int complement = 0;
 
-/* When non-zero, this flag causes GNU tr to provide strict
+/* When nonzero, this flag causes GNU tr to provide strict
    compliance with POSIX draft 1003.2.11.2.  The POSIX spec
    says that when -d is used without -s, string2 (if present)
    must be ignored.  Silently ignoring arguments is a bad idea.
    The default GNU behavior is to give a usage message and exit.
-   Additionally, when this flag is non-zero, tr prints warnings
+   Additionally, when this flag is nonzero, tr prints warnings
    on stderr if it is being used in a manner that is not portable.
    Applicable warnings are given by default, but are suppressed
    if the environment variable `POSIXLY_CORRECT' is set, since
    being POSIX conformant means we can't issue such messages.
    Warnings on the following topics are suppressed when this
-   variable is non-zero:
+   variable is nonzero:
    1. Ambiguous octal escapes.  */
 static int posix_pedantic;
 
@@ -246,7 +275,7 @@ static int posix_pedantic;
    corresponding character in string2.  That is, string1 is effectively
    truncated to the length of string2.
 
-   When non-zero, this flag causes GNU tr to imitate the behavior
+   When nonzero, this flag causes GNU tr to imitate the behavior
    of System V tr when translating with string1 longer than string2.
    The default is to emulate BSD tr.  This flag is ignored in modes where
    no translation is performed.  Emulating the System V tr
@@ -301,10 +330,10 @@ static SET_TYPE in_delete_set[N_CHARS];
    two specification strings and the delete switch is not given.  */
 static char xlate[N_CHARS];
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output then exit.  */
+/* If nonzero, print the version on standard output then exit.  */
 static int show_version;
 
 static struct option const long_options[] =
@@ -388,7 +417,7 @@ translation or deletion.\n\
   exit (status);
 }
 
-/* Return non-zero if the character C is a member of the
+/* Return nonzero if the character C is a member of the
    equivalence class containing the character EQUIV_CLASS.  */
 
 static int
@@ -397,7 +426,7 @@ is_equiv_class_member (unsigned int equiv_class, unsigned int c)
   return (equiv_class == c);
 }
 
-/* Return non-zero if the character C is a member of the
+/* Return nonzero if the character C is a member of the
    character class CHAR_CLASS.  */
 
 static int
@@ -453,16 +482,24 @@ is_char_class_member (enum Char_class char_class, unsigned int c)
 /* Perform the first pass over each range-spec argument S, converting all
    \c and \ddd escapes to their one-byte representations.  The conversion
    is done in-place, so S must point to writable storage.  If an invalid
-   quote sequence is found print an error message and return non-zero.
+   quote sequence is found print an error message and return nonzero.
    Otherwise set *LEN to the length of the resulting string and return
    zero.  The resulting array of characters may contain zero-bytes;
    however, on input, S is assumed to be null-terminated, and hence
    cannot contain actual (non-escaped) zero bytes.  */
 
 static int
-unquote (unsigned char *s, size_t *len)
+unquote (const unsigned char *s, struct E_string *es)
 {
   size_t i, j;
+  size_t len;
+
+  len = strlen ((char *) s);
+
+  es->s = (unsigned char *) xmalloc (len);
+  es->escaped = (int *) xmalloc (len * sizeof (es->escaped[0]));
+  for (i = 0; i < len; i++)
+    es->escaped[i] = 0;
 
   j = 0;
   for (i = 0; s[i]; i++)
@@ -542,18 +579,26 @@ unquote (unsigned char *s, size_t *len)
 	      return 1;
 
 	    default:
-	      error (0, 0, _("invalid backslash escape `\\%c'"), s[i + 1]);
-	      return 1;
+	      if (posix_pedantic)
+		{
+		  error (0, 0, _("invalid backslash escape `\\%c'"), s[i + 1]);
+		  return 1;
+		}
+	      else
+	        {
+		  c = s[i + 1];
+		  es->escaped[j] = 1;
+		}
 	    }
 	  ++i;
-	  s[j++] = c;
+	  es->s[j++] = c;
 	  break;
 	default:
-	  s[j++] = s[i];
+	  es->s[j++] = s[i];
 	  break;
 	}
     }
-  *len = j;
+  es->len = j;
   return 0;
 }
 
@@ -675,7 +720,7 @@ append_normal_char (struct Spec_list *list, unsigned int c)
 
 /* Append a newly allocated structure representing the range
    of characters from FIRST to LAST to the specification list LIST.
-   Return non-zero if LAST precedes FIRST in the collating sequence,
+   Return nonzero if LAST precedes FIRST in the collating sequence,
    zero otherwise.  This means that '[c-c]' is acceptable.  */
 
 static int
@@ -709,23 +754,18 @@ append_range (struct Spec_list *list, unsigned int first, unsigned int last)
 /* If CHAR_CLASS_STR is a valid character class string, append a
    newly allocated structure representing that character class to the end
    of the specification list LIST and return 0.  If CHAR_CLASS_STR is not
-   a valid string, print an error message and return non-zero.  */
+   a valid string return nonzero.  */
 
 static int
-append_char_class (struct Spec_list *list, const unsigned char *char_class_str, size_t len)
+append_char_class (struct Spec_list *list,
+		   const unsigned char *char_class_str, size_t len)
 {
   enum Char_class char_class;
   struct List_element *new;
 
   char_class = look_up_char_class (char_class_str, len);
   if (char_class == CC_NO_CLASS)
-    {
-      char *tmp = make_printable_str (char_class_str, len);
-
-      error (0, 0, _("invalid character class `%s'"), tmp);
-      free (tmp);
-      return 1;
-    }
+    return 1;
   new = (struct List_element *) xmalloc (sizeof (struct List_element));
   new->next = NULL;
   new->type = RE_CHAR_CLASS;
@@ -742,7 +782,8 @@ append_char_class (struct Spec_list *list, const unsigned char *char_class_str,
    is a non-negative repeat count.  */
 
 static void
-append_repeated_char (struct Spec_list *list, unsigned int the_char, size_t repeat_count)
+append_repeated_char (struct Spec_list *list, unsigned int the_char,
+		      size_t repeat_count)
 {
   struct List_element *new;
 
@@ -760,22 +801,16 @@ append_repeated_char (struct Spec_list *list, unsigned int the_char, size_t repe
    the length of that string, LEN, if LEN is exactly one, append
    a newly allocated structure representing the specified
    equivalence class to the specification list, LIST and return zero.
-   If LEN is not 1, issue an error message and return non-zero.  */
+   If LEN is not 1, return nonzero.  */
 
 static int
-append_equiv_class (struct Spec_list *list, const unsigned char *equiv_class_str, size_t len)
+append_equiv_class (struct Spec_list *list,
+		    const unsigned char *equiv_class_str, size_t len)
 {
   struct List_element *new;
 
   if (len != 1)
-    {
-      char *tmp = make_printable_str (equiv_class_str, len);
-
-      error (0, 0, _("%s: equivalence class operand must be a single character"),
-	     tmp);
-      free (tmp);
-      return 1;
-    }
+    return 1;
   new = (struct List_element *) xmalloc (sizeof (struct List_element));
   new->next = NULL;
   new->type = RE_EQUIV_CLASS;
@@ -809,16 +844,18 @@ substr (const unsigned char *p, size_t first_idx, size_t last_idx)
 /* Search forward starting at START_IDX for the 2-char sequence
    (PRE_BRACKET_CHAR,']') in the string P of length P_LEN.  If such
    a sequence is found, set *RESULT_IDX to the index of the first
-   character and return non-zero. Otherwise return zero.  P may contain
+   character and return nonzero. Otherwise return zero.  P may contain
    zero bytes.  */
 
 static int
-find_closing_delim (const unsigned char *p, size_t start_idx, size_t p_len, unsigned int pre_bracket_char, size_t *result_idx)
+find_closing_delim (const struct E_string *es, size_t start_idx,
+		    unsigned int pre_bracket_char, size_t *result_idx)
 {
   size_t i;
 
-  for (i = start_idx; i < p_len - 1; i++)
-    if (p[i] == pre_bracket_char && p[i + 1] == ']')
+  for (i = start_idx; i < es->len - 1; i++)
+    if (es->s[i] == pre_bracket_char && es->s[i + 1] == ']'
+	&& !es->escaped[i] && !es->escaped[i + 1])
       {
 	*result_idx = i;
 	return 1;
@@ -830,7 +867,7 @@ find_closing_delim (const unsigned char *p, size_t start_idx, size_t p_len, unsi
    containing embedded zero bytes, to a long integer value.
    If the string represents a negative value, a value larger
    than LONG_MAX, or if all LEN characters do not represent a
-   valid integer, return non-zero and do not modify *VAL.
+   valid integer, return nonzero and do not modify *VAL.
    Otherwise, return zero and set *VAL to the converted value.  */
 
 static int
@@ -879,22 +916,24 @@ non_neg_strtol (const unsigned char *s, size_t len, size_t *val)
    and return -2.  */
 
 static int
-find_bracketed_repeat (const unsigned char *p, size_t start_idx, size_t p_len, unsigned int *char_to_repeat, size_t *repeat_count, size_t *closing_bracket_idx)
+find_bracketed_repeat (const struct E_string *es, size_t start_idx,
+		       unsigned int *char_to_repeat, size_t *repeat_count,
+		       size_t *closing_bracket_idx)
 {
   size_t i;
 
-  assert (start_idx + 1 < p_len);
-  if (p[start_idx + 1] != '*')
+  assert (start_idx + 1 < es->len);
+  if (!ES_MATCH (es, start_idx + 1, '*'))
     return -1;
 
-  for (i = start_idx + 2; i < p_len; i++)
+  for (i = start_idx + 2; i < es->len; i++)
     {
-      if (p[i] == ']')
+      if (ES_MATCH (es, i, ']'))
 	{
 	  const unsigned char *digit_str;
 	  size_t digit_str_len = i - start_idx - 2;
 
-	  *char_to_repeat = p[start_idx];
+	  *char_to_repeat = es->s[start_idx];
 	  if (digit_str_len == 0)
 	    {
 	      /* We've matched [c*] -- no explicit repeat count.  */
@@ -905,8 +944,9 @@ find_bracketed_repeat (const unsigned char *p, size_t start_idx, size_t p_len, u
 
 	  /* Here, we have found [c*s] where s should be a string
 	     of octal or decimal digits.  */
-	  digit_str = &p[start_idx + 2];
-	  if (non_neg_strtol (digit_str, digit_str_len, repeat_count))
+	  digit_str = &es->s[start_idx + 2];
+	  if (non_neg_strtol (digit_str, digit_str_len, repeat_count)
+	      || *repeat_count > BEGIN_STATE)
 	    {
 	      char *tmp = make_printable_str (digit_str, digit_str_len);
 	      error (0, 0, _("invalid repeat count `%s' in [c*n] construct"),
@@ -921,6 +961,30 @@ find_bracketed_repeat (const unsigned char *p, size_t start_idx, size_t p_len, u
   return -1;			/* No bracket found.  */
 }
 
+/* Return nonzero if the string at ES->s[IDX] matches the regular
+   expression `\*[0-9]*\]', zero otherwise.  To match, the `*' and
+   the `]' must not be escaped.  */
+
+static int
+star_digits_closebracket (const struct E_string *es, int idx)
+{
+  int i;
+
+  if (!ES_MATCH (es, idx, '*'))
+    return 0;
+
+  for (i = idx + 1; i < es->len; i++)
+    {
+      if (!ISDIGIT (es->s[i]))
+	{
+	  if (ES_MATCH (es, i, ']'))
+	    return 1;
+	  return 0;
+	}
+    }
+  return 0;
+}
+
 /* Convert string UNESACPED_STRING (which has been preprocessed to
    convert backslash-escape sequences) of length LEN characters into
    a linked list of the following 5 types of constructs:
@@ -934,12 +998,12 @@ find_bracketed_repeat (const unsigned char *p, size_t start_idx, size_t p_len, u
       - c Any other character is interpreted as itself.  */
 
 static int
-build_spec_list (const unsigned char *unescaped_string, size_t len, struct Spec_list *result)
+build_spec_list (const struct E_string *es, struct Spec_list *result)
 {
   const unsigned char *p;
   size_t i;
 
-  p = unescaped_string;
+  p = es->s;
 
   /* The main for-loop below recognizes the 4 multi-character constructs.
      A character that matches (in its context) none of the multi-character
@@ -948,95 +1012,140 @@ build_spec_list (const unsigned char *unescaped_string, size_t len, struct Spec_
      less are composed solely of normal characters.  Hence, the index of
      the outer for-loop runs only as far as LEN-2.  */
 
-  for (i = 0; i + 2 < len;)
+  for (i = 0; i + 2 < es->len; /* empty */)
     {
-      switch (p[i])
+      if (ES_MATCH (es, i, '['))
 	{
-	  int fall_through;
+	  int matched_multi_char_construct;
+	  size_t closing_bracket_idx;
+	  unsigned int char_to_repeat;
+	  size_t repeat_count;
 	  int err;
 
-	case '[':
-	  fall_through = 0;
-	  switch (p[i + 1])
+	  matched_multi_char_construct = 1;
+	  if (ES_MATCH (es, i + 1, ':')
+	      || ES_MATCH (es, i + 1, '='))
 	    {
 	      size_t closing_delim_idx;
-	      size_t closing_bracket_idx;
-	      unsigned int char_to_repeat;
-	      size_t repeat_count;
 	      int found;
 
-	    case ':':
-	    case '=':
-	      found = find_closing_delim (p, i + 2, len, p[i + 1],
+	      found = find_closing_delim (es, i + 2, p[i + 1],
 					  &closing_delim_idx);
 	      if (found)
 		{
 		  int parse_failed;
 		  unsigned char *opnd_str = substr (p, i + 2,
 						    closing_delim_idx - 1);
+		  size_t opnd_str_len = closing_delim_idx - 1 - (i + 2) + 1;
+
 		  if (p[i + 1] == ':')
-		    parse_failed = append_char_class (result, opnd_str,
-				     (closing_delim_idx - 1) - (i + 2) + 1);
+		    {
+		      parse_failed = append_char_class (result, opnd_str,
+							opnd_str_len);
+
+		      /* FIXME: big comment.  */
+		      if (parse_failed)
+			{
+			  if (star_digits_closebracket (es, i + 2))
+			    {
+			      free (opnd_str);
+			      goto try_bracketed_repeat;
+			    }
+			  else
+			    {
+			      char *tmp = make_printable_str (opnd_str,
+							      opnd_str_len);
+			      error (0, 0, _("invalid character class `%s'"),
+				     tmp);
+			      free (tmp);
+			      return 1;
+			    }
+			}
+		    }
 		  else
-		    parse_failed = append_equiv_class (result, opnd_str,
-				     (closing_delim_idx - 1) - (i + 2) + 1);
+		    {
+		      parse_failed = append_equiv_class (result, opnd_str,
+							 opnd_str_len);
+
+		      /* FIXME: big comment.  */
+		      if (parse_failed)
+			{
+			  if (star_digits_closebracket (es, i + 2))
+			    {
+			      free (opnd_str);
+			      goto try_bracketed_repeat;
+			    }
+			  else
+			    {
+			      char *tmp = make_printable_str (opnd_str,
+							      opnd_str_len);
+			      error (0, 0,
+	       _("%s: equivalence class operand must be a single character"),
+				     tmp);
+			      free (tmp);
+			      return 1;
+			    }
+			}
+		    }
 		  free (opnd_str);
 
-		  /* Return non-zero if append_*_class reports a problem.  */
+		  /* Return nonzero if append_*_class reports a problem.  */
 		  if (parse_failed)
 		    return 1;
 		  else
 		    i = closing_delim_idx + 2;
-		  break;
+		  continue;
 		}
 	      /* Else fall through.  This could be [:*] or [=*].  */
-	    default:
-	      /* Determine whether this is a bracketed repeat range
-	         matching the RE \[.\*(dec_or_oct_number)?\].  */
-	      err = find_bracketed_repeat (p, i + 1, len, &char_to_repeat,
-					   &repeat_count,
-					   &closing_bracket_idx);
-	      if (err == 0)
-		{
-		  append_repeated_char (result, char_to_repeat, repeat_count);
-		  i = closing_bracket_idx + 1;
-		  break;
-		}
-	      else if (err == -1)
-		{
-		  fall_through = 1;
-		}
-	      else
-		/* Found a string that looked like [c*n] but the
-		   numeric part was invalid.  */
-		return 1;
-	      break;
 	    }
-	  if (!fall_through)
-	    break;
 
-	  /* Here if we've tried to match [c*n], [:str:], and [=c=]
-	     and none of them fit.  So we still have to consider the
-	     range `[-c' (from `[' to `c').  */
-	default:
-	  /* Look ahead one char for ranges like a-z.  */
-	  if (p[i + 1] == '-')
+	try_bracketed_repeat:
+
+	  /* Determine whether this is a bracketed repeat range
+	     matching the RE \[.\*(dec_or_oct_number)?\].  */
+	  err = find_bracketed_repeat (es, i + 1, &char_to_repeat,
+				       &repeat_count,
+				       &closing_bracket_idx);
+	  if (err == 0)
 	    {
-	      if (append_range (result, p[i], p[i + 2]))
-		return 1;
-	      i += 3;
+	      append_repeated_char (result, char_to_repeat, repeat_count);
+	      i = closing_bracket_idx + 1;
+	    }
+	  else if (err == -1)
+	    {
+	      matched_multi_char_construct = 0;
 	    }
 	  else
 	    {
-	      append_normal_char (result, p[i]);
-	      ++i;
+	      /* Found a string that looked like [c*n] but the
+		 numeric part was invalid.  */
+	      return 1;
 	    }
-	  break;
+
+	  if (matched_multi_char_construct)
+	    continue;
+
+	  /* We reach this point if P does not match [:str:], [=c=],
+	     [c*n], or [c*].  Now, see if P looks like a range `[-c'
+	     (from `[' to `c').  */
+	}
+
+      /* Look ahead one char for ranges like a-z.  */
+      if (ES_MATCH (es, i + 1, '-'))
+	{
+	  if (append_range (result, p[i], p[i + 2]))
+	    return 1;
+	  i += 3;
+	}
+      else
+	{
+	  append_normal_char (result, p[i]);
+	  ++i;
 	}
     }
 
-  /* Now handle the (2 or fewer) remaining characters p[i]..p[len - 1].  */
-  for (; i < len; i++)
+  /* Now handle the (2 or fewer) remaining characters p[i]..p[es->len - 1].  */
+  for (; i < es->len; i++)
     append_normal_char (result, p[i]);
 
   return 0;
@@ -1314,16 +1423,16 @@ spec_init (struct Spec_list *spec_list)
    one converts all \c and \ddd escapes to their one-byte representations.
    The second constructs a linked specification list, SPEC_LIST, of the
    characters and constructs that comprise the argument string.  If either
-   of these passes detects an error, this function returns non-zero.  */
+   of these passes detects an error, this function returns nonzero.  */
 
 static int
-parse_str (unsigned char *s, struct Spec_list *spec_list)
+parse_str (const unsigned char *s, struct Spec_list *spec_list)
 {
-  size_t len;
+  struct E_string es;
 
-  if (unquote (s, &len))
+  if (unquote (s, &es))
     return 1;
-  if (build_spec_list (s, len, spec_list))
+  if (build_spec_list (&es, spec_list))
     return 1;
   return 0;
 }
@@ -1433,7 +1542,8 @@ appear in\n\tstring2 are `upper' and `lower'"));
 
       if (s2->n_indefinite_repeats > 1)
 	{
-	  error (1, 0, _("only one [c*] repeat construct may appear in string2"));
+	  error (1, 0,
+		 _("only one [c*] repeat construct may appear in string2"));
 	}
 
       if (translating)
@@ -1659,7 +1769,7 @@ read_and_xlate (unsigned char *buf, long int size, PFI not_used)
 /* Initialize a boolean membership set IN_SET with the character
    values obtained by traversing the linked list of constructs S
    using the function `get_next'.  If COMPLEMENT_THIS_SET is
-   non-zero the resulting set is complemented.  */
+   nonzero the resulting set is complemented.  */
 
 static void
 set_initialize (struct Spec_list *s, int complement_this_set, SET_TYPE *in_set)
@@ -1765,7 +1875,8 @@ without squeezing repeats"));
     }
 
   if (squeeze_repeats && non_option_args == 0)
-    error (1, 0, _("at least one string must be given when squeezing repeats"));
+    error (1, 0,
+	   _("at least one string must be given when squeezing repeats"));
 
   spec_init (s1);
   if (parse_str ((unsigned char *) argv[optind], s1))
