@@ -58,13 +58,6 @@
 char *xmalloc ();
 char *xrealloc ();
 
-static FILE *next_file ();
-static void add_tabstop ();
-static void parse_tabstops ();
-static void unexpand ();
-static void usage ();
-static void validate_tabstops ();
-
 /* The name this program was run with. */
 char *program_name;
 
@@ -99,10 +92,10 @@ static int have_read_stdin;
 /* Status to return to the system. */
 static int exit_status;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output then exit.  */
+/* If nonzero, print the version on standard output then exit.  */
 static int show_version;
 
 static struct option const longopts[] =
@@ -114,90 +107,24 @@ static struct option const longopts[] =
   {NULL, 0, NULL, 0}
 };
 
-void
-main (argc, argv)
-     int argc;
-     char **argv;
+/* Add tab stop TABVAL to the end of `tab_list', except
+   if TABVAL is -1, do nothing. */
+
+static void
+add_tabstop (int tabval)
 {
-  int tabval = -1;		/* Value of tabstop being read, or -1. */
-  int c;			/* Option character. */
-
-  have_read_stdin = 0;
-  exit_status = 0;
-  convert_entire_line = 0;
-  tab_list = NULL;
-  first_free_tab = 0;
-  program_name = argv[0];
-
-  while ((c = getopt_long (argc, argv, "at:,0123456789", longopts, (int *) 0))
-	 != EOF)
-    {
-      switch (c)
-	{
-	case 0:
-	  break;
-
-	case '?':
-	  usage (1);
-	case 'a':
-	  convert_entire_line = 1;
-	  break;
-	case 't':
-	  convert_entire_line = 1;
-	  parse_tabstops (optarg);
-	  break;
-	case ',':
-	  add_tabstop (tabval);
-	  tabval = -1;
-	  break;
-	default:
-	  if (tabval == -1)
-	    tabval = 0;
-	  tabval = tabval * 10 + c - '0';
-	  break;
-	}
-    }
-
-  if (show_version)
-    {
-      printf ("unexpand - %s\n", version_string);
-      exit (0);
-    }
-
-  if (show_help)
-    usage (0);
-
-  add_tabstop (tabval);
-
-  validate_tabstops (tab_list, first_free_tab);
-
-  if (first_free_tab == 0)
-    tab_size = 8;
-  else if (first_free_tab == 1)
-    tab_size = tab_list[0];
-  else
-    tab_size = 0;
-
-  if (optind == argc)
-    file_list = stdin_argv;
-  else
-    file_list = &argv[optind];
-
-  unexpand ();
-
-  if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, "-");
-  if (fclose (stdout) == EOF)
-    error (1, errno, _("write error"));
-  exit (exit_status);
+  if (tabval == -1)
+    return;
+  if (first_free_tab % TABLIST_BLOCK == 0)
+    tab_list = (int *) xrealloc (tab_list, first_free_tab + TABLIST_BLOCK);
+  tab_list[first_free_tab++] = tabval;
 }
 
 /* Add the comma or blank separated list of tabstops STOPS
    to the list of tabstops. */
 
 static void
-parse_tabstops (stops)
-     char *stops;
+parse_tabstops (const char *stops)
 {
   int tabval = -1;
 
@@ -221,27 +148,11 @@ parse_tabstops (stops)
   add_tabstop (tabval);
 }
 
-/* Add tab stop TABVAL to the end of `tab_list', except
-   if TABVAL is -1, do nothing. */
-
-static void
-add_tabstop (tabval)
-     int tabval;
-{
-  if (tabval == -1)
-    return;
-  if (first_free_tab % TABLIST_BLOCK == 0)
-    tab_list = (int *) xrealloc (tab_list, first_free_tab + TABLIST_BLOCK);
-  tab_list[first_free_tab++] = tabval;
-}
-
 /* Check that the list of tabstops TABS, with ENTRIES entries,
    contains only nonzero, ascending values. */
 
 static void
-validate_tabstops (tabs, entries)
-     int *tabs;
-     int entries;
+validate_tabstops (const int *tabs, int entries)
 {
   int prev_tab = 0;
   int i;
@@ -256,11 +167,58 @@ validate_tabstops (tabs, entries)
     }
 }
 
+/* Close the old stream pointer FP if it is non-NULL,
+   and return a new one opened to read the next input file.
+   Open a filename of `-' as the standard input.
+   Return NULL if there are no more input files.  */
+
+static FILE *
+next_file (FILE *fp)
+{
+  static char *prev_file;
+  char *file;
+
+  if (fp)
+    {
+      if (ferror (fp))
+	{
+	  error (0, errno, "%s", prev_file);
+	  exit_status = 1;
+	}
+      if (fp == stdin)
+	clearerr (fp);		/* Also clear EOF. */
+      else if (fclose (fp) == EOF)
+	{
+	  error (0, errno, "%s", prev_file);
+	  exit_status = 1;
+	}
+    }
+
+  while ((file = *file_list++) != NULL)
+    {
+      if (file[0] == '-' && file[1] == '\0')
+	{
+	  have_read_stdin = 1;
+	  prev_file = file;
+	  return stdin;
+	}
+      fp = fopen (file, "r");
+      if (fp)
+	{
+	  prev_file = file;
+	  return fp;
+	}
+      error (0, errno, "%s", file);
+      exit_status = 1;
+    }
+  return NULL;
+}
+
 /* Change spaces to tabs, writing to stdout.
    Read each file in `file_list', in order. */
 
 static void
-unexpand ()
+unexpand (void)
 {
   FILE *fp;			/* Input stream. */
   int c;			/* Each input character. */
@@ -391,57 +349,8 @@ unexpand ()
     }
 }
 
-/* Close the old stream pointer FP if it is non-NULL,
-   and return a new one opened to read the next input file.
-   Open a filename of `-' as the standard input.
-   Return NULL if there are no more input files.  */
-
-static FILE *
-next_file (fp)
-     FILE *fp;
-{
-  static char *prev_file;
-  char *file;
-
-  if (fp)
-    {
-      if (ferror (fp))
-	{
-	  error (0, errno, "%s", prev_file);
-	  exit_status = 1;
-	}
-      if (fp == stdin)
-	clearerr (fp);		/* Also clear EOF. */
-      else if (fclose (fp) == EOF)
-	{
-	  error (0, errno, "%s", prev_file);
-	  exit_status = 1;
-	}
-    }
-
-  while ((file = *file_list++) != NULL)
-    {
-      if (file[0] == '-' && file[1] == '\0')
-	{
-	  have_read_stdin = 1;
-	  prev_file = file;
-	  return stdin;
-	}
-      fp = fopen (file, "r");
-      if (fp)
-	{
-	  prev_file = file;
-	  return fp;
-	}
-      error (0, errno, "%s", file);
-      exit_status = 1;
-    }
-  return NULL;
-}
-
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
@@ -467,3 +376,79 @@ Instead of -t NUMBER or -t LIST, -NUMBER or -LIST may be used.\n\
     }
   exit (status);
 }
+
+void
+main (int argc, char **argv)
+{
+  int tabval = -1;		/* Value of tabstop being read, or -1. */
+  int c;			/* Option character. */
+
+  have_read_stdin = 0;
+  exit_status = 0;
+  convert_entire_line = 0;
+  tab_list = NULL;
+  first_free_tab = 0;
+  program_name = argv[0];
+
+  while ((c = getopt_long (argc, argv, "at:,0123456789", longopts, (int *) 0))
+	 != EOF)
+    {
+      switch (c)
+	{
+	case 0:
+	  break;
+
+	case '?':
+	  usage (1);
+	case 'a':
+	  convert_entire_line = 1;
+	  break;
+	case 't':
+	  convert_entire_line = 1;
+	  parse_tabstops (optarg);
+	  break;
+	case ',':
+	  add_tabstop (tabval);
+	  tabval = -1;
+	  break;
+	default:
+	  if (tabval == -1)
+	    tabval = 0;
+	  tabval = tabval * 10 + c - '0';
+	  break;
+	}
+    }
+
+  if (show_version)
+    {
+      printf ("unexpand - %s\n", version_string);
+      exit (0);
+    }
+
+  if (show_help)
+    usage (0);
+
+  add_tabstop (tabval);
+
+  validate_tabstops (tab_list, first_free_tab);
+
+  if (first_free_tab == 0)
+    tab_size = 8;
+  else if (first_free_tab == 1)
+    tab_size = tab_list[0];
+  else
+    tab_size = 0;
+
+  if (optind == argc)
+    file_list = stdin_argv;
+  else
+    file_list = &argv[optind];
+
+  unexpand ();
+
+  if (have_read_stdin && fclose (stdin) == EOF)
+    error (1, errno, "-");
+  if (fclose (stdout) == EOF)
+    error (1, errno, _("write error"));
+  exit (exit_status);
+}
