@@ -25,21 +25,29 @@
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
+
+#if HAVE_LIMITS_H
+# include <limits.h>
+#endif
+
+#ifndef UINT_MAX
+# define UINT_MAX ((unsigned int) ~(unsigned int) 0)
+#endif
+
+#ifndef INT_MAX
+# define INT_MAX ((int) (UINT_MAX >> 1))
+#endif
+
 #include "system.h"
 #include "linebuffer.h"
 #include "version.h"
 #include "error.h"
+#include "xstrtol.h"
 
 /* Undefine, to avoid warning about redefinition on some systems.  */
 #undef min
 #define min(x, y) ((x) < (y) ? (x) : (y))
 
-static char *find_field ();
-static int different ();
-static void check_file ();
-static void usage ();
-static void writeline ();
-
 /* The name this program was run with. */
 char *program_name;
 
@@ -72,10 +80,10 @@ enum output_mode
 /* Which lines to output. */
 static enum output_mode mode;
 
-/* If non-zero, display usage information and exit.  */
+/* If nonzero, display usage information and exit.  */
 static int show_help;
 
-/* If non-zero, print the version on standard output then exit.  */
+/* If nonzero, print the version on standard output then exit.  */
 static int show_version;
 
 static struct option const longopts[] =
@@ -90,109 +98,113 @@ static struct option const longopts[] =
   {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
-
-void
-main (argc, argv)
-     int argc;
-     char *argv[];
+
+static void
+usage (int status)
 {
-  int optc;
-  char *infile = "-", *outfile = "-";
-
-  program_name = argv[0];
-  skip_chars = 0;
-  skip_fields = 0;
-  check_chars = 0;
-  mode = output_all;
-  countmode = count_none;
-
-  while ((optc = getopt_long (argc, argv, "0123456789cdf:s:uw:", longopts,
-			      (int *) 0)) != EOF)
+  if (status != 0)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
+  else
     {
-      switch (optc)
-	{
-	case 0:
-	  break;
+      printf (_("\
+Usage: %s [OPTION]... [INPUT [OUTPUT]]\n\
+"),
+	      program_name);
+      printf (_("\
+Discard all but one of successive identical lines from INPUT (or\n\
+standard input), writing to OUTPUT (or standard output).\n\
+\n\
+  -c, --count           prefix lines by the number of occurrences\n\
+  -d, --repeated        only print duplicate lines\n\
+  -f, --skip-fields=N   avoid comparing the first N fields\n\
+  -s, --skip-chars=N    avoid comparing the first N characters\n\
+  -u, --unique          only print unique lines\n\
+  -w, --check-chars=N   compare no more than N characters in lines\n\
+  -N                    same as -f N\n\
+  +N                    same as -s N\n\
+      --help            display this help and exit\n\
+      --version         output version information and exit\n\
+\n\
+A field is a run of whitespace, than non-whitespace characters.\n\
+Fields are skipped before chars. \n\
+"));
+    }
+  exit (status);
+}
 
-	case '0':
-	case '1':
-	case '2':
-	case '3':
-	case '4':
-	case '5':
-	case '6':
-	case '7':
-	case '8':
-	case '9':
-	  skip_fields = skip_fields * 10 + optc - '0';
-	  break;
+/* Given a linebuffer LINE,
+   return a pointer to the beginning of the line's field to be compared. */
 
-	case 'c':
-	  countmode = count_occurrences;
-	  break;
+static char *
+find_field (const struct linebuffer *line)
+{
+  register int count;
+  register char *lp = line->buffer;
+  register int size = line->length;
+  register int i = 0;
 
-	case 'd':
-	  mode = output_repeated;
-	  break;
-
-	case 'f':		/* Like '-#'. */
-	  skip_fields = atoi (optarg);
-	  break;
-
-	case 's':		/* Like '+#'. */
-	  skip_chars = atoi (optarg);
-	  break;
-
-	case 'u':
-	  mode = output_unique;
-	  break;
-
-	case 'w':
-	  check_chars = atoi (optarg);
-	  break;
-
-	default:
-	  usage (1);
-	}
+  for (count = 0; count < skip_fields && i < size; count++)
+    {
+      while (i < size && ISBLANK (lp[i]))
+	i++;
+      while (i < size && !ISBLANK (lp[i]))
+	i++;
     }
 
-  if (show_version)
+  for (count = 0; count < skip_chars && i < size; count++)
+    i++;
+
+  return lp + i;
+}
+
+/* Return zero if two strings OLD and NEW match, nonzero if not.
+   OLD and NEW point not to the beginnings of the lines
+   but rather to the beginnings of the fields to compare.
+   OLDLEN and NEWLEN are their lengths. */
+
+static int
+different (const char *old, const char *new, int oldlen, int newlen)
+{
+  register int order;
+
+  if (check_chars)
     {
-      printf ("uniq - %s\n", version_string);
-      exit (0);
+      if (oldlen > check_chars)
+	oldlen = check_chars;
+      if (newlen > check_chars)
+	newlen = check_chars;
     }
-
-  if (show_help)
-    usage (0);
-
-  if (optind >= 2 && strcmp (argv[optind - 1], "--") != 0)
-    {
-      /* Interpret non-option arguments with leading `+' only
-	 if we haven't seen `--'.  */
-      while (optind < argc && argv[optind][0] == '+')
-	skip_chars = atoi (argv[optind++]);
-    }
-
-  if (optind < argc)
-    infile = argv[optind++];
-
-  if (optind < argc)
-    outfile = argv[optind++];
-
-  if (optind < argc)
-    usage (1);			/* Extra arguments. */
-
-  check_file (infile, outfile);
-
-  exit (0);
+  order = memcmp (old, new, min (oldlen, newlen));
+  if (order == 0)
+    return oldlen - newlen;
+  return order;
 }
 
+/* Output the line in linebuffer LINE to stream STREAM
+   provided that the switches say it should be output.
+   If requested, print the number of times it occurred, as well;
+   LINECOUNT + 1 is the number of times that the line occurred. */
+
+static void
+writeline (const struct linebuffer *line, FILE *stream, int linecount)
+{
+  if ((mode == output_unique && linecount != 0)
+      || (mode == output_repeated && linecount == 0))
+    return;
+
+  if (countmode == count_occurrences)
+    fprintf (stream, "%7d\t", linecount + 1);
+
+  fwrite (line->buffer, sizeof (char), line->length, stream);
+  putc ('\n', stream);
+}
+
 /* Process input file INFILE with output to OUTFILE.
    If either is "-", use the standard I/O stream for it instead. */
 
 static void
-check_file (infile, outfile)
-     char *infile, *outfile;
+check_file (const char *infile, const char *outfile)
 {
   FILE *istream;
   FILE *ostream;
@@ -260,113 +272,126 @@ check_file (infile, outfile)
   free (lb1.buffer);
   free (lb2.buffer);
 }
-
-/* Given a linebuffer LINE,
-   return a pointer to the beginning of the line's field to be compared. */
 
-static char *
-find_field (line)
-     struct linebuffer *line;
+void
+main (int argc, char **argv)
 {
-  register int count;
-  register char *lp = line->buffer;
-  register int size = line->length;
-  register int i = 0;
+  int optc;
+  char *infile = "-", *outfile = "-";
 
-  for (count = 0; count < skip_fields && i < size; count++)
+  program_name = argv[0];
+  skip_chars = 0;
+  skip_fields = 0;
+  check_chars = 0;
+  mode = output_all;
+  countmode = count_none;
+
+  while ((optc = getopt_long (argc, argv, "0123456789cdf:s:uw:", longopts,
+			      (int *) 0)) != EOF)
     {
-      while (i < size && ISBLANK (lp[i]))
-	i++;
-      while (i < size && !ISBLANK (lp[i]))
-	i++;
+      switch (optc)
+	{
+	case 0:
+	  break;
+
+	case '0':
+	case '1':
+	case '2':
+	case '3':
+	case '4':
+	case '5':
+	case '6':
+	case '7':
+	case '8':
+	case '9':
+	  skip_fields = skip_fields * 10 + optc - '0';
+	  break;
+
+	case 'c':
+	  countmode = count_occurrences;
+	  break;
+
+	case 'd':
+	  mode = output_repeated;
+	  break;
+
+	case 'f':		/* Like '-#'. */
+	  {
+	    long int tmp_long;
+	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
+		|| tmp_long <= 0 || tmp_long > INT_MAX)
+	      error (1, 0, _("invalid number of fields to skip: `%s'"),
+		     optarg);
+	    skip_fields = (int) tmp_long;
+	  }
+	  break;
+
+	case 's':		/* Like '+#'. */
+	  {
+	    long int tmp_long;
+	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
+		|| tmp_long <= 0 || tmp_long > INT_MAX)
+	      error (1, 0, _("invalid number of bytes to skip: `%s'"),
+		     optarg);
+	    skip_chars = (int) tmp_long;
+	  }
+	  break;
+
+	case 'u':
+	  mode = output_unique;
+	  break;
+
+	case 'w':
+	  {
+	    long int tmp_long;
+	    if (xstrtol (optarg, NULL, 10, &tmp_long, NULL) != LONGINT_OK
+		|| tmp_long <= 0 || tmp_long > INT_MAX)
+	      error (1, 0, _("invalid number of bytes to compare: `%s'"),
+		     optarg);
+	    check_chars = (int) tmp_long;
+	  }
+	  break;
+
+	default:
+	  usage (1);
+	}
     }
 
-  for (count = 0; count < skip_chars && i < size; count++)
-    i++;
-
-  return lp + i;
-}
-
-/* Return zero if two strings OLD and NEW match, nonzero if not.
-   OLD and NEW point not to the beginnings of the lines
-   but rather to the beginnings of the fields to compare.
-   OLDLEN and NEWLEN are their lengths. */
-
-static int
-different (old, new, oldlen, newlen)
-     char *old;
-     char *new;
-     int oldlen;
-     int newlen;
-{
-  register int order;
-
-  if (check_chars)
+  if (show_version)
     {
-      if (oldlen > check_chars)
-	oldlen = check_chars;
-      if (newlen > check_chars)
-	newlen = check_chars;
+      printf ("uniq - %s\n", version_string);
+      exit (0);
     }
-  order = memcmp (old, new, min (oldlen, newlen));
-  if (order == 0)
-    return oldlen - newlen;
-  return order;
-}
-
-/* Output the line in linebuffer LINE to stream STREAM
-   provided that the switches say it should be output.
-   If requested, print the number of times it occurred, as well;
-   LINECOUNT + 1 is the number of times that the line occurred. */
 
-static void
-writeline (line, stream, linecount)
-     struct linebuffer *line;
-     FILE *stream;
-     int linecount;
-{
-  if ((mode == output_unique && linecount != 0)
-      || (mode == output_repeated && linecount == 0))
-    return;
+  if (show_help)
+    usage (0);
 
-  if (countmode == count_occurrences)
-    fprintf (stream, "%7d\t", linecount + 1);
-
-  fwrite (line->buffer, sizeof (char), line->length, stream);
-  putc ('\n', stream);
-}
-
-static void
-usage (status)
-     int status;
-{
-  if (status != 0)
-    fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
-  else
+  if (optind >= 2 && strcmp (argv[optind - 1], "--") != 0)
     {
-      printf (_("\
-Usage: %s [OPTION]... [INPUT [OUTPUT]]\n\
-"),
-	      program_name);
-      printf (_("\
-Discard all but one of successive identical lines from INPUT (or\n\
-standard input), writing to OUTPUT (or standard output).\n\
-\n\
-  -c, --count           prefix lines by the number of occurrences\n\
-  -d, --repeated        only print duplicate lines\n\
-  -f, --skip-fields=N   avoid comparing the N first fields\n\
-  -s, --skip-chars=N    avoid comparing the N first characters\n\
-  -u, --unique          only print unique lines\n\
-  -w, --check-chars=N   compare no more than N characters in lines\n\
-  -N                    same as -f N\n\
-  +N                    same as -s N\n\
-      --help            display this help and exit\n\
-      --version         output version information and exit\n\
-\n\
-A field is a run of whitespace, than non-whitespace characters.\n\
-Fields are skipped before chars. \n\
-"));
+      /* Interpret non-option arguments with leading `+' only
+	 if we haven't seen `--'.  */
+      while (optind < argc && argv[optind][0] == '+')
+	{
+	  char *opt_str = argv[optind++];
+	  long int tmp_long;
+	  if (xstrtol (opt_str, NULL, 10, &tmp_long, NULL) != LONGINT_OK
+	      || tmp_long <= 0 || tmp_long > INT_MAX)
+	    error (1, 0, _("invalid number of bytes to compare: `%s'"),
+		   opt_str);
+	  skip_chars = (int) tmp_long;
+	}
     }
-  exit (status);
+
+  if (optind < argc)
+    infile = argv[optind++];
+
+  if (optind < argc)
+    outfile = argv[optind++];
+
+  if (optind < argc)
+    usage (1);			/* Extra arguments. */
+
+  check_file (infile, outfile);
+
+  exit (0);
 }
